from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_03.StructureManagement import SearchForClustersResponse, CreateReuseAssemblyResp, AlignAssemblyData, SearchForClusters, ConfigurationsInfo, ConfigureMultipleStructuresResponse, CreateReuseAssembly, GetClusterDetails
from tcsoa.gen.Internal.Manufacturing._2015_03.Attachments import AttachmentResponse, CreateAttachmentsInput, AttachmentLines, AttachmentsInput
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2015_03.StructureSearch import GetSearchCrieriaFromRecipeResp
from tcsoa.gen.Internal.Manufacturing._2015_03.ResourceManagement import ImportStep3DModels2Response, UnzipGtcPackageResponse, ImportStepP21Files3Response, MapClassificationObjectResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def getSearchCriteriaFromRecipe(cls, recipes: List[BusinessObject]) -> GetSearchCrieriaFromRecipeResp:
        """
        This operation retrieves the search criteria stored in a recipe on a Manufacturing BOM (MBOM) node. The
        returned search critieria will be used to automatically resolve Engineering BOM (EBOM) lines under the MBOM
        node.
        
        Exceptions:
        >This operation will throw a ServiceException if the Teamcenter database schema does not have the recipe
        constructs.
        - 300376 The schema element Mfg0MEMBOMRecipe is missing, and is needed to be able to store the recipe. Please
        contact your system administrator to ensure that the Foundation template is properly installed.
        
        """
        return cls.execute_soa_method(
            method_name='getSearchCriteriaFromRecipe',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureSearch',
            params={'recipes': recipes},
            response_cls=GetSearchCrieriaFromRecipeResp,
        )


class ResourceManagementService(TcService):

    @classmethod
    def importStep3DModels2(cls, icoIds: List[str]) -> ImportStep3DModels2Response:
        """
        This operation imports STEP 3D model files for Generic Tool Catalog (GTC) Packages. The STEP files are
        converted to NX part and to JT file format and imported into Teamcenter. If needed, items are created for the
        Classification objects (ICO). One UGMaster dataset for the NX part file and one DirectModel dataset for the JT
        file is created below the (new) item. 
        
        
        Use cases:
        There are two different use case s:
            A) The specified ICO is classified in a vendor catalog class
        (the ICO has an attribute -159003 "3D Model file name")
            B) The specified ICO is classified in an MRL MyComponents class
        (the ICO does not have an attribute -159003 "3D Model file name")
        
        In use case A, this operation retrieves the file name of the 3D model directly from GTC attribute 
        -159003 "3D Model file name".
        (Another attribute ID instead of -159003 can be defined in the "MRMGTC3DModelAttributeID" preference.)
        
        In use case B, the SOA operation checks if there is an Manufacturing Resource Library (MRL) attribute -40930
        "Vendor Reference Object ID" in the given ICO. This attribute is used to store the reference from the MRL
        MyComponents ICO to the GTC vendor catalog ICO. If this attribute exists in the ICO's class and has a valid ICO
        ID as value, the 3D model file name will be retrieved from the referenced ICO. (see use case A)
        (Another attribute ID instead of -40930 can be defined in the "MRMGTCReferenceObjectAttributeID"preference.)
        
        The SOA operation retrieves the information about the server-sided directory, where all STEP 3D models of the
        catalog are stored, based on the class of the GTC vendor catalog ICO.
        
        If there is a problem during importing one of the 3D models, the operation will continue importing the 3D
        models of the following ICOs.
        
        This operation is very similar to importStep3DModels(). The difference is that this new operation returns in
        addition to the service data also information about the ICO and the newly created item.
        
        Note: The Graphics Builder has to be configured properly for this operation to work.
        """
        return cls.execute_soa_method(
            method_name='importStep3DModels2',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='ResourceManagement',
            params={'icoIds': icoIds},
            response_cls=ImportStep3DModels2Response,
        )

    @classmethod
    def importStepP21Files3(cls, classId: str, catalogRootDirectory: str, importOption: int) -> ImportStepP21Files3Response:
        """
        This operation imports STEP P21 files containing product data pertaining to Generic Tool Catalog (GTC) Packages
        into the Classification classes that represent vendor catalogs inside the Teamcenter database. It creates
        Internal Classification objects (ICOs) in those classes, and associated data that represents the products.
        Depending on the contents of the GTC Package, it may also create associated items, item revisions, datasets,
        and associated files to store drawings and images that further describe those tool components.
        """
        return cls.execute_soa_method(
            method_name='importStepP21Files3',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='ResourceManagement',
            params={'classId': classId, 'catalogRootDirectory': catalogRootDirectory, 'importOption': importOption},
            response_cls=ImportStepP21Files3Response,
        )

    @classmethod
    def mapClassificationObject(cls, sourceIcoId: str, targetItemId: str, targetItemName: str, targetItemTypeName: str, targetItemRevId: str, targetItemDescription: str, targetClassId: str, options: int) -> MapClassificationObjectResponse:
        """
        This operation maps an Internal Classification Object  (ICO)  from its Classification class into the class
        identified by the provided targetClassId. It will also create an item of the provided item type, having the
        provided IDs, name, and description created. This operation maps the attribute values from the source ICO into
        the target ICO based on the mapping rules defined in Classification Admin. Using the options parameter the user
        can decide if datasets from a source item should be copied to the tartget item.
        
        Use cases:
        A typical use case when this operation can be used is the mapping of ICOs from the Generic Tool Catalog (GTC)
        classes into the Manufacturing Resource Library's (MRL) tool component classes. The GTC classes contain catalog
        tool components. In the MRL tool component classes are those tool components that are actively used in the
        customer's shopfloor.
        In the Classification Admin application are mapping rules (Mapping Views) defined that control which GTC class
        is mapped on what MRL class and which GTC attribute is mapped on what MRL attribute.
        During the mapping operation a new item and ICO are created in the MRL class and the attribute values from the
        source ICO are transferred into the target ICO. If the source component has a dataset attached, this operation
        allows to copy the dataset to the new target component. 
        """
        return cls.execute_soa_method(
            method_name='mapClassificationObject',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='ResourceManagement',
            params={'sourceIcoId': sourceIcoId, 'targetItemId': targetItemId, 'targetItemName': targetItemName, 'targetItemTypeName': targetItemTypeName, 'targetItemRevId': targetItemRevId, 'targetItemDescription': targetItemDescription, 'targetClassId': targetClassId, 'options': options},
            response_cls=MapClassificationObjectResponse,
        )

    @classmethod
    def unzipGtcPackage(cls, transientFmsZipFileTicket: str) -> UnzipGtcPackageResponse:
        """
        This operation uses the File Management Service (FMS) transient file ticket provided to unzip an uploaded
        Generic Tool Catalog (GTC) package ZIP file on the Teamcenter server. The ZIP file will be extracted to:
        $TC_DATA/offset-directory-specified-in-the-MRMGTCPackageServerDir-system-preference
        In that directory, a new directory with the ZIP file name is created. If this directory already exists, a new
        directory with the ZIP file name and an appended timestamp is created.
        
        Example:
        Environment Variable TC_DATA:        $TC_DATA
        Preference MRMGTCPackageServerDir:    ResourceManagement\GTCPackages
        GTC Package ZIP Filename:            sampleGTCPackage.zip
        Resulting Directory:        
        $TC_DATA\ResourceManagement\GTCPackages\sampleGTCPackage  
        
        """
        return cls.execute_soa_method(
            method_name='unzipGtcPackage',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='ResourceManagement',
            params={'transientFmsZipFileTicket': transientFmsZipFileTicket},
            response_cls=UnzipGtcPackageResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def configureMultipleStructures(cls, configurationsInfo: List[ConfigurationsInfo]) -> ConfigureMultipleStructuresResponse:
        """
        The operation applies the input configurations to each input BOMWindow and its root BOMLine or BOPLine object
        of the structure. The type of BOPLine object can be Mfg0BvrProcess. The configurations those are applicable to
        a BOMWindow in a Teamcenter session are used as input to configure the structures. The input configuration
        rules supported by the operation are as follows -
        a)    Variant rule 
        b)    Revision rule
        c)    Effectivity 
        d)    Show or hide the BOMLine or BOPLine objects configured-out by variant rule configuration on the BOMWindow.
        e)    Show or hide the BOMLine or BOPLine objects configured-out by the occurrence effectivity configuration on
        the BOMWindow.
        f)    Show or hide the suppressed BOMLine or BOPLine objects in the BOMWindow.
        g)    Show or hide the BOMLine or BOPLine objects configured-out by the incremental changes configuration on
        the BOMWindow.
        h)    Show or hide BOPLine objects configured-out by the assigned occurrences in the BOMWindow.
        """
        return cls.execute_soa_method(
            method_name='configureMultipleStructures',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'configurationsInfo': configurationsInfo},
            response_cls=ConfigureMultipleStructuresResponse,
        )

    @classmethod
    def searchForClusters(cls, input: List[SearchForClusters]) -> SearchForClustersResponse:
        """
        This operation creates clusters(groups) of objects (currently, BOMLine objects) from an Engineering BOM (EBOM )
        matching the pattern of currently assigned objects from the same EBOM under a given Model assembly (currently
        represented by a BOMLine object) in a Manufacturing BOM (MBOM)). The pattern to create the clusters is based on
        combination of item id and relative transform differences betweeen the assigned objects in the model assembly.
        If the transforms do not uniquely identify the patterns (example: all transforms are Identity) the operation
        may identify a very large set as pattern. The normal usage for this operation should be when the transforms
        form well defined patterns. It is assumed that the properties of BOMLine objects that form the cluster are not
        used after this operation returns. Calling this operation itself will not return the properties of the BOMLine
        objects that are contained in the clusters. It is imperative that the response of this operation will be used
        as input to another subsequent operation getClusterDetails to get the properties for the BOMLine objects found
        in each cluster.
        """
        return cls.execute_soa_method(
            method_name='searchForClusters',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=SearchForClustersResponse,
        )

    @classmethod
    def createReuseAssemblies(cls, input: List[CreateReuseAssembly]) -> CreateReuseAssemblyResp:
        """
        This operation creates reuse  assemblies based on BOMLine objects under a template model assembly defined in
        the Manufacturing BOM (MBOM) and the cluster of BOMLine objects returned from the operation searchForClusters.
        Each of the assemblies created can be reused as a model assembly. The searchForClusters operation returns 
        collections of BOMLine objects from Engineering BOM (EBOM) based on the template model assembly.
        """
        return cls.execute_soa_method(
            method_name='createReuseAssemblies',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=CreateReuseAssemblyResp,
        )

    @classmethod
    def alignAssemblies(cls, input: List[AlignAssemblyData]) -> ServiceData:
        """
        This operation aligns the "bl_abs_occ_id" property and the absolute transform of a BOMLine object in source
        structure to a BOMLine object in a target structure. In addition, if alignMode is not specified, any lines with
        matching relative transforms to these aligned lines in their respective structures will have the
        "bl_abs_occ_id" property aligned too. The operation also allows for aligning only "bl_abs_occ_id" property
        without changing the transform or aligning BOMLine properties based on the preference "MEAlignedPropertiesList".
        """
        return cls.execute_soa_method(
            method_name='alignAssemblies',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getClusterDetails(cls, input: List[GetClusterDetails]) -> ServiceData:
        """
        This operation gets the properties for the BOMLine objects returned from a previous call to  searchForClusters.
        It is imperative that the response of this operation will be used as input to this operation to get the
        properties for the BOMLine objects found in each cluster.
        """
        return cls.execute_soa_method(
            method_name='getClusterDetails',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=ServiceData,
        )


class AttachmentsService(TcService):

    @classmethod
    def createAttachmentLines(cls, attachmentsInputList: List[CreateAttachmentsInput]) -> AttachmentResponse:
        """
        This service operation creates attachment line representing a form or dataset. The operation takes place in
        multiple steps as below.
        - Dataset  is created.
        - Highest revision of the newly created Dataset is fetched.
        - The write file ticket for the file to be associated with the Dataset  iis fetched.
        - The file is committed with the Dataset.
        - An attachment line is created for the Dataset.
        
        
        The attachment line created by this operation is a run time business object. In client use case scenario this
        object should be used with precaution as it gets destroyed when another BOMLine is selected in BOMWindow.
        Note that this operation doesn't take care of uploading the write file ticket to FMS server thus preventing any
        edit of the file.
        
        Use cases:
        Use Case 1: Creating new dataset line in attachment view.
        This operation can be used to create a new dataset line in the attachment window. The newly created dataset
        will be related to the ItemRevision of the BOMLine with the specified relation.
        
        Use Case 2: Creating new form line in attachment view.
        This operation can be used to create a new attachment line representing a Form. The newly created form will be
        related to the ItemRevision of the BOMLine with a default relation like IMAN_specifications.
        """
        return cls.execute_soa_method(
            method_name='createAttachmentLines',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='Attachments',
            params={'attachmentsInputList': attachmentsInputList},
            response_cls=AttachmentResponse,
        )

    @classmethod
    def getAttachmentLines(cls, inputList: List[AttachmentsInput]) -> AttachmentLines:
        """
        This service operation fetches all the attachment lines for the given BOMLine object.  Using this operation
        saves at least 3 server calls in case of attachment window use cases. 
        
        Use cases:
        Use Case 1: User opens the attachment window/panel.
        This operation can be used to get the attachment lines for the selected BOMLine object  and display in
        attachment window/panel. In this case only the BOMLine object is to be sent to the operation without any
        attachment window objects. A new attachment window is created in this case.
        
        Use Case 2: User selects another BOMLine while the attachment window/panel is opened.
        While the attachment panel is open, this operation can be used to get the attachment lines on change of
        selection to other BOMLine. In this case the CfgAttachmentWindow objects of the opened attachment window(s)
        is(are) to be sent to the operation along with the BOMLine object. No new attachment window is created in this
        case. The given BOMLine is used to set the top line(s) of the attachment window(s) provided to the operation.
        """
        return cls.execute_soa_method(
            method_name='getAttachmentLines',
            library='Internal-Manufacturing',
            service_date='2015_03',
            service_name='Attachments',
            params={'inputList': inputList},
            response_cls=AttachmentLines,
        )



from __future__ import annotations

from tcsoa.gen.Core._2007_09.ProjectLevelSecurity import AssignedOrRemovedObjects
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2007_09.DataManagement import ExpandGRMRelationsPref2, ExpandGRMRelationsResponse2, RemoveNamedReferenceFromDatasetInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def loadObjects(cls, uids: List[str]) -> ServiceData:
        """
        Load business objects into the client data model for each of the UIDs supplied. The business objects are loaded
        from the Teamcenter server's in memory cache of business objects or from the database. UIDs of runtime business
        objects (BOMLines) that are not currently loaded in the Teamcenter server's memory will result in a partial
        error being returned.
        """
        return cls.execute_soa_method(
            method_name='loadObjects',
            library='Core',
            service_date='2007_09',
            service_name='DataManagement',
            params={'uids': uids},
            response_cls=ServiceData,
        )

    @classmethod
    def removeNamedReferenceFromDataset(cls, inputs: List[RemoveNamedReferenceFromDatasetInfo]) -> ServiceData:
        """
        This operation removes the specified named references from a dataset.
        
        If the 'NamedReferenceInfo.targetObject' input is not specified then all named references of the type specified
        are removed from the dataset.  If the 'NamedReferenceInfo.targetObject' input is specified then only that named
        reference is removed from the dataset.  If the 'NamedReferenceInfo.deleteTarget' input is true then the
        'NamedReferenceInfo.targetObject' will be deleted if it is no longer referenced.
        
        
        Use cases:
        User deletes a single named reference from a dataset that has multiple named references of the same type.
        
        For this operation, the dataset is passed in along with the named reference type and object reference for the
        specific named reference to be removed from the dataset.  The specific named reference is removed from the
        dataset and the dataset is added to the 'ServiceData' list of updated objects.
        """
        return cls.execute_soa_method(
            method_name='removeNamedReferenceFromDataset',
            library='Core',
            service_date='2007_09',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def expandGRMRelationsForPrimary(cls, primaryObjects: List[BusinessObject], pref: ExpandGRMRelationsPref2) -> ExpandGRMRelationsResponse2:
        """
        This operation returns the secondary objects that are related to the input primary objects.  Relation type
        names and secondary object types can be input as a filter to reduce the set of returned secondary objects.  In
        the context of expanding primary objects, secondary objects may be referred to as side objects. If no relation
        type names or secondary object types are input, then all related objects will be returned.
        
        For improved performance, if an item is the output of the expansion, then its associated item revisions and the
        datasets for those item revisions will be returned.  Similarly, if an item revision is the output of the
        expansion, then its associated datasets will be returned.
        
        
        Use cases:
        User wants to see all the secondary objects related to the input primary item object.
        
        For this operation, the item object is input in 'primaryObjects' and the filter preference 'info'
        'relationTypeName' and 'otherSideObjectTypes' parameters are set to be empty.  All secondary objects that have
        a relation to the item are returned in 'ExpandGRMRelationsOutput2'  'relationshipData'.  The primary object,
        secondary objects and relation objects are returned as plain objects in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='expandGRMRelationsForPrimary',
            library='Core',
            service_date='2007_09',
            service_name='DataManagement',
            params={'primaryObjects': primaryObjects, 'pref': pref},
            response_cls=ExpandGRMRelationsResponse2,
        )

    @classmethod
    def expandGRMRelationsForSecondary(cls, secondaryObjects: List[BusinessObject], pref: ExpandGRMRelationsPref2) -> ExpandGRMRelationsResponse2:
        """
        This operation returns the primary objects that are related to the input secondary objects.  Relation type
        names and secondary object types can be input as a filter to reduce the set of returned primary objects.  In
        the context of expanding secondary objects, primary objects may be referred to as side objects.  If no relation
        type names or secondary object types are input, then all related objects will be returned.
        
        For improved performance, if an item is the output of the expansion, then its associated item revisions and the
        datasets for those item revisions will be returned.  Similarly, if an item revision is the output of the
        expansion, then its associated datasets will be returned.
        
        
        Use cases:
        User wants to see all the primary objects related to the input secondary dataset object.
        
        For this operation, the dataset object is input in 'secondaryObjects' and the filter preference info
        'relationTypeName' and 'otherSideObjectTypes' parameters are set to be empty.  All primary objects that have a
        relation to the dataset are returned in 'ExpandGRMRelationsOutput2'  'relationshipData'.  The secondary object,
        primary objects and relation objects are returned as plain objects in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='expandGRMRelationsForSecondary',
            library='Core',
            service_date='2007_09',
            service_name='DataManagement',
            params={'secondaryObjects': secondaryObjects, 'pref': pref},
            response_cls=ExpandGRMRelationsResponse2,
        )


class ProjectLevelSecurityService(TcService):

    @classmethod
    def assignOrRemoveObjects(cls, assignedOrRemovedobjects: List[AssignedOrRemovedObjects]) -> ServiceData:
        """
        This operation assigns the given set of workspace objects to the given projects. Similarly, it removes an
        additional set of given workspace objects from the same set of given projects. If user is not privileged to
        assign objects to any of the given projects then this operation will return the error 101014 : You have
        insufficient privilege to assign object to a project. Similarly, if user is not privileged to remove objects
        from any of the given projects then this operation will return error 101015: You have insufficient privilege to
        remove object from a project.  These errors will not terminate processing of rest of the objects.
        """
        return cls.execute_soa_method(
            method_name='assignOrRemoveObjects',
            library='Core',
            service_date='2007_09',
            service_name='ProjectLevelSecurity',
            params={'assignedOrRemovedobjects': assignedOrRemovedobjects},
            response_cls=ServiceData,
        )



from tcsoa.gen.Internal_Multisite._2017_11.services import ArchiveRestoreService as imp0
from tcsoa.gen.Internal_Multisite._2018_06.services import SearchService as imp1
from tcsoa.gen.Internal_Multisite._2020_12.services import ImportExportTCXMLService as imp2
from tcsoa.gen.Internal_Multisite._2011_06.services import ImportExportAsyncService as imp3
from tcsoa.utils import TcService


class ArchiveRestoreService(TcService):
    archiveObjects = imp0.archiveObjects
    getReportFileTicket = imp0.getReportFileTicket
    restoreObjects = imp0.restoreObjects


class SearchService(TcService):
    fetchOdsRecords = imp1.fetchOdsRecords


class ImportExportTCXMLService(TcService):
    getMultisiteDashBoardData = imp2.getMultisiteDashBoardData


class ImportExportAsyncService(TcService):
    remoteExport = imp3.remoteExport



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_10.StructureManagement import PasteOrReplaceAssemblyInContextInfo, AlignLinesInBOMResponse, GetEquivalentPropValuesResp, AlignLinesInBOMData, GetEquivalentPropValuesElement, PasteOrReplaceAssemblyInContextResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2015_10.StructureSearch import FindEquivalentLinesResp, FindCPCResponse, FindEquivalentLinesIn
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def pasteOrReplaceAssemblyInContext(cls, assemblyInContextInput: List[PasteOrReplaceAssemblyInContextInfo]) -> PasteOrReplaceAssemblyInContextResponse:
        """
        This operation pastes an assembly item under a target assembly Item or replaces an existing assembly Item with
        a new assembly Item while retaining any in-context edits on the properties and attachments under the target
        assembly.  For the paste, the new assembly Item can be either a previously saved raw (with no child lines)
        Item, or the same assembly Item. For replace, the new assembly Item is always a saved raw assembly Item ( no
        child lines). A cloning rule controls which child lines and in-context attachments are to be cloned or
        referenced under the new assembly's BOMLine.
        
        Use cases:
        A user wants to reuse an existing assembly with occurrence effectivity 1-10, but wants the new assembly to have
        all the original in-context edits and attachments and then modify occurrence effectivity to 11-UP. User either
        wants the new assembly to be a new object or the same object itself which will be pasted under a target object.
        If user doesn't want the old item any more user can also replace the existing assembly with a new object. The
        new object or the same object ( in case of pasting the same object) will be modified according to a cloning
        rule before pasting onto the target. Then user can set new effectivities on original and the new assembly. 
        The cloning rule is specified by a preference "CopyAssemblyInContextTemplates" which points to another
        preference e.g. "Item.Item.CopyAssemblyInContext" implying that the source is of type "Item" and target is of
        type "Item". It specifies information on what objects to clone or reference. It will also include rules for any
        in-context attachments.
        Example clauses on the cloning rule are below.. Wildcard "*" refers to all objects:
        *:*:Attribute.items_tag:Clone
        *:*:Attribute.structure_revisions:Clone
        *:*:Attribute.bom_view:Clone
        *:*:Attribute.parent_item:Clone
        *:*:OccType.*:Reference
        *:*:Relation.*:Reference
        class.Folder:*:Attribute.contents:Reference
        """
        return cls.execute_soa_method(
            method_name='pasteOrReplaceAssemblyInContext',
            library='Internal-Manufacturing',
            service_date='2015_10',
            service_name='StructureManagement',
            params={'assemblyInContextInput': assemblyInContextInput},
            response_cls=PasteOrReplaceAssemblyInContextResponse,
        )

    @classmethod
    def alignLinesInBOM(cls, alignInputs: List[AlignLinesInBOMData]) -> AlignLinesInBOMResponse:
        """
        This service operation aligns ID in Context (Top Level) and Absolute Transformation Matrix between an
        Engineering BOM (EBOM) and a Manufacturing BOM (MBOM). The operation also optionally aligns occurrence
        properties based on the preference MEAlignedPropertiesList.
        
        Use cases:
        Case 1: Single line alignment
        Given a selected source BOMLine object in a structure (e.g., EBOM) and its equivalent target BOMLine object in
        another structure (e.g., MBOM), this operation aligns these two lines.
        
        Case 2: Assemblies alignment
        Given a list of source scopes (e.g., in EBOM) and a list of target scopes from non-reused assemblies (e.g., in
        MBOM), this operation traverses the specified scopes and aligns the equivalent scope lines and all equivalent
        BOMLine objects below the scopes. 
        
        Case 3: Reused assemblies alignment
        Given a list of source scopes (e.g., in EBOM) and a list of target scopes from reused assemblies (e.g., in
        MBOM), this operation traverses the specified scopes and aligns all equivalent BOMLine objects below the scopes.
        """
        return cls.execute_soa_method(
            method_name='alignLinesInBOM',
            library='Internal-Manufacturing',
            service_date='2015_10',
            service_name='StructureManagement',
            params={'alignInputs': alignInputs},
            response_cls=AlignLinesInBOMResponse,
        )

    @classmethod
    def getEquivalentPropertyValues(cls, input: List[GetEquivalentPropValuesElement]) -> GetEquivalentPropValuesResp:
        """
        This operation finds equivalent property values on an ItemRevision for a given list of input properties.  The 
        operation receives sets of property names associated with either an Item, ItemRevision, Item Master or Item
        Revision Master, an original ItemRevision and new Item Revision.    For each property it looks for an
        equivalent property for the original ItemRevision. If found, then its value is retrieved and returned in a map
        of input property names to equivalent property values. In addition it returns the attachment relation and
        secondary object  that match cloning rules.
        """
        return cls.execute_soa_method(
            method_name='getEquivalentPropertyValues',
            library='Internal-Manufacturing',
            service_date='2015_10',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=GetEquivalentPropValuesResp,
        )


class StructureSearchService(TcService):

    @classmethod
    def findCollabPlanningContext(cls, inputScopes: List[BusinessObject]) -> FindCPCResponse:
        """
        This service operation searches and returns all the Collaboration Planning Contexts (CPCs) that refer to the
        given business objects. CPC is not a MECollaborationContext object, and is a term used for mix production.
        
        Use cases:
        - Use Case 1: You need to know what CPCs were created from a certain data (in the original structure like EBOP
        or WorkArea). You should than select the original data and search for CPCs that refer to this data.
        - Use Case 2: You can select any object i.e. Mfg0BvrPlantBOP/
        Mfg0BvrProcess/Mfg0BvrOperation/Mfg0BvrWorkarea/Mfg0BvrProcessLine/Mfg0BvrProcessStatio/ Mfg0BvrProcessArea and
        search for the CPC that contains it.
        
        """
        return cls.execute_soa_method(
            method_name='findCollabPlanningContext',
            library='Internal-Manufacturing',
            service_date='2015_10',
            service_name='StructureSearch',
            params={'inputScopes': inputScopes},
            response_cls=FindCPCResponse,
        )

    @classmethod
    def findEquivalentLines(cls, searchInputs: List[FindEquivalentLinesIn]) -> FindEquivalentLinesResp:
        """
        This operation finds equivalent lines between Engineering BOM (EBOM) and Manufacturing BOM (MBOM) using search
        criteria such as ID in Context (Top Level), Item ID and Absolute Transformation Matrix, or ID in Context (All
        Levels).
        
        Use cases:
        Case 1: Find equivalent lines in the other structure
        Given a list of BOMLine objects in one structure (e.g., MBOM), this operation will find their equivalent
        BOMLine objects in the other structure (e.g., EBOM), based on specified search criteria. In this case, the
        equivalent BOMLine objects of the source will be searched under the target scopes. The return will be a map
        between source BOMLine objects and their equivalent target BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='findEquivalentLines',
            library='Internal-Manufacturing',
            service_date='2015_10',
            service_name='StructureSearch',
            params={'searchInputs': searchInputs},
            response_cls=FindEquivalentLinesResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InsertDatasetVersionArgs(TcBaseObj):
    """
    The InsertDatasetVersionArgs struct is used to pass in multiple sets of data
    to be used in a single call.  These structs are passed in the collection of
    input arguments to the function insertDatasetVersion.
    
    :var dataset: The Dataset of you would like insert a version.
    :var versionNumber: The version number you would like to insert.
    """
    dataset: BusinessObject = None
    versionNumber: int = 0


@dataclass
class InsertDatasetVersionResponse(TcBaseObj):
    """
    The InsertDatasetVersionResponse struct is the output values for the call to
    the function insertDatasetVersion.
    
    :var datasetTags: A two(2) element array where the first element represents the Dataset version which was inserted
    and the second element representing the zero Dataset.
    :var svcData: The service data needed to pass the above values back.
    """
    datasetTags: List[BusinessObject] = ()
    svcData: ServiceData = None


@dataclass
class QueryTranslationRequestsArgs(TcBaseObj):
    """
    The QueryTranslationRequestsArgs is the possible data to be used in querying for Translation
    Request objects.  These values can all be set, some of them, etc.
    
    If none of the values are set, then 0 items will be returned and the query
    will be skipped.
    
    :var providers: A list of providers.
    :var services: A list of services (i.e. translators)
    :var states: A list of states (i.e. INITIAL, TERMINAL)
    :var priorities: A list of priorities (i.e. 1,2,3)
    :var modifiedDate: A modified before date.  This is only a single date and
    not a list of them because that would create a range and
    not a before date.
    :var primaryObjects: The primary objects (i.e. Datasets)
    :var taskIDs: taskIDs
    """
    providers: List[str] = ()
    services: List[str] = ()
    states: List[str] = ()
    priorities: List[int] = ()
    modifiedDate: str = ''
    primaryObjects: List[BusinessObject] = ()
    taskIDs: List[str] = ()


@dataclass
class QueryTranslationRequestsOutput(TcBaseObj):
    """
    The QueryTranslationRequestsOutput struct is the output structure containing the requests
    that were returned from the corresponding input arguments to the query.
    
    :var queriedRequests: A list of the ETSTranslationRequests returned from the query within the Teamcenter database.
    """
    queriedRequests: List[BusinessObject] = ()


@dataclass
class QueryTranslationRequestsResponse(TcBaseObj):
    """
    The QueryTranslationRequestsResponse struct is the output values for the call to
    the function queryTranslationRequests.
    
    :var outputObjects: A list of the output data associated with this call.
    :var svcData: he service data needed to pass the above values back.
    """
    outputObjects: List[QueryTranslationRequestsOutput] = ()
    svcData: ServiceData = None


@dataclass
class UpdateTranslationRequestArgs(TcBaseObj):
    """
    The UpdateTranslationRequestArgs is the possible data to be used in updating a
    Translation Request object.
    
    :var requestToUpdate: The Translation Request to update.  This value MUST be
    set and can not be empty.
    :var priority: The priority to assign to the request. (0 LOW to 3 HIGH)
    :var currentState: The current state of the request.  This value is used as a safety check when setting the
    nextState of the request to make sure we have the current up to date request.
    :var nextState: The next state to assign to the request.
    
    Valid states include: COMPLETE, DUPLICATE, DELETE, CANCELLED, SUPERSEDED, NO_TRANS, PREPARING, SCHEDULED,
    SUPERSEDING, TRANSLATING, LOADING
    
    :var taskID: The task ID to assign to the request.
    :var translatorArgs: The translator arguments to assign to the request.  This will override the current translator
    arguments and is not append to the current list.
    """
    requestToUpdate: BusinessObject = None
    priority: int = 0
    currentState: str = ''
    nextState: str = ''
    taskID: str = ''
    translatorArgs: List[str] = ()


@dataclass
class UpdateTranslationRequestOutput(TcBaseObj):
    """
    The UpdateTranslationRequestOutput struct is the output structure containing the request
    that was updated and a boolean value stating if the request was updated.
    
    :var requestUpdated: This is the request that was updated and should be the
    request that the client was previously referencing.
    :var wasRequestUpdated: This is a boolean representing whether or not the request
    was successfully updated.
    """
    requestUpdated: BusinessObject = None
    wasRequestUpdated: bool = False


@dataclass
class UpdateTranslationRequestResponse(TcBaseObj):
    """
    The UpdateTranslationRequestResponse struct is the output values for the call to
    the function updateTranslationRequest.
    
    :var outputObjects: A list of the output data associated with this call.
    :var svcData: The service data needed to pass the above values back.
    """
    outputObjects: List[UpdateTranslationRequestOutput] = ()
    svcData: ServiceData = None


@dataclass
class CreateDatasetOfVersionArgs(TcBaseObj):
    """
    The CreateDatasetOfVersionArgs is used to carry the data needed to create a Dataset of a particular version.  This
    is similar to the core create Dataset method with the exception that this method takes a specific version and
    creates the Dataset at that version even if it is the first instance of that Dataset.
    
    :var itemRevision: After creation, any release status assigned to this ItemRevision will be copied to the Dataset.
    :var datasetName: The name of the Dataset to create.
    :var typeName: The type of the Dataset to create.
    :var tool: The Tool to associate with the new Dataset.
    :var version: The version to assign to the new Dataset.
    """
    itemRevision: ItemRevision = None
    datasetName: str = ''
    typeName: str = ''
    tool: BusinessObject = None
    version: int = 0


@dataclass
class CreateDatasetOfVersionResponse(TcBaseObj):
    """
    The CreateDatasetOfVersionResponse structure is the output values for the call to the function
    createDatasetOfVersion.
    
    :var createdDatasets: A list of the Datasets created corresponding to the list of input data.
    :var svcData: The service data needed to pass the above values back.
    """
    createdDatasets: List[Dataset] = ()
    svcData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Core._2014_10.DataManagement import SaveAsIn, GetPasteRelationsResponse2, GenerateIdsResponse, ChildrenInputData, DeepCopyDataInput, GenerateIdInput, GetDeepCopyDataResponse
from tcsoa.gen.Core._2013_05.DataManagement import GetPasteRelationsInputData
from typing import List
from tcsoa.gen.BusinessObjects import POM_object
from tcsoa.gen.Core._2014_10.ProjectLevelSecurity import PropagateDataElement
from tcsoa.gen.Core._2012_09.DataManagement import RelateInfoIn
from tcsoa.gen.Core._2011_06.DataManagement import SaveAsObjectsResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def propagateData(cls, propagateDataElements: List[PropagateDataElement]) -> None:
        """
        This operation can propagate  security or anyother teamcenter data based on the source object and applicable
        propagation rules. The propagateData operation should only be invoked when propagation rule is configured to
        run in background. 
        The propagation rule defined in BMIDE states what data from source to desination object has to be
        propagated.For example project_list, license_list from Item to ItemRevision.
        
         The term "security data" is an abbreviated form of any of the following methods for applying additional
        security settings on an object.
        - Assigning a Project(s) to an object
        - Removing object from a Project(s)
        - Attaching an ADA license(s) to an object
        - Detaching an ADA license(s) from an object
        - Setting any of the classification attributes:
        - o    ip_classification
        - o    itar_classification
        - < other attributes > Note that customers can add their own attributes and configure them in the BMIDE to be
        propagated along with the ip_classification, and itar_classification attributes.
        
        
        
        Use cases:
        Use Case: Propagate security data from Cpd0CollaborativeDesign to Cp0DesignElement.
        
        Propagation rule for use case :
        Source type                -- Cpd0CollaborativeDesign.
        Direction                    -- Reverse.
        Destination type        -- Cp0DesignElement.
        Operation                -- All.
        property name            -- mdl0model_object.
        Property group            -- Security group I.
        Traversal condition    -- 4GDtraversalcondition.
        Propagation condition -- 4GDpropagationcondition.
        Style                        -- Merge.
        Background                -- TRUE.
        
        Property Group for use case: 
        Property group name             -- Security group I.
        List of properties in a group -- project_list, licence_list.
        
        For this use case the propagation rule and the property group shown in the above two tables respectively are
        applicable.
        Here the values from the properties project_list and the license_list would be propagated from
        Cpd0CollaborativeDesign to Cp0DesignElement through the operation propagateData.
        
        The above use case is one data propagation example. As part of this project (023479-Project ADA propagation for
        CPD) we are adding propagation rules into 4GD and foundation template. Customers can also add more rules.
        """
        return cls.execute_soa_method(
            method_name='propagateData',
            library='Core',
            service_date='2014_10',
            service_name='ProjectLevelSecurity',
            params={'propagateDataElements': propagateDataElements},
            response_cls=None,
        )


class DataManagementService(TcService):

    @classmethod
    def pruneNamedReferences(cls, namedReferences: List[POM_object]) -> ServiceData:
        """
        This operation performs a prune operation by a given list of Dataset 'named references', per the following
        criteria
        1. Remove the input named references from their owning Dataset
        2. Delete the input 'named reference' objects
        3. Delete the owning Dataset objects which contain no' named references' after the prune operation
        4. The pruned 'named references', deleted Datasets and updated Datasets will be returned 
        """
        return cls.execute_soa_method(
            method_name='pruneNamedReferences',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'namedReferences': namedReferences},
            response_cls=ServiceData,
        )

    @classmethod
    def addChildren(cls, inputData: List[ChildrenInputData]) -> ServiceData:
        """
        This operation adds a list of objects as children to a list of parent objects which could be related by
        relation or reference properties. If the property name is not supplied as input it will use the default
        relation property between the parent and the children given by
        <ParentTypeName>_<ChildTypeName>_default_relation.
        Please see the Preferences and Environment variables reference in the Rich client interface guide for
        information on configuring these preferences.
        
        
        Use cases:
        Add MSWord object as target attachments to EPMTask object.
        
        Use AddChildren operation and provide EPMTask as the parentObj, MSWord object as the childrenObj and
        target_attachments as the property name. Also, provide clientId value to identify this add operation.
        Here, the target_attachments property is a runtime property. The AddChildren operation internally will modify
        two properties attachments and attachment_types which are saved in the database.
        """
        return cls.execute_soa_method(
            method_name='addChildren',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'inputData': inputData},
            response_cls=ServiceData,
        )

    @classmethod
    def removeChildren(cls, inputData: List[ChildrenInputData]) -> ServiceData:
        """
        This operation removes a list of objects as children to a list of parent objects which could be related by
        relation or reference properties. If the property name is not supplied as input it will use the default
        relation property between the parent and the children given by ParentTypeName>_ChildTypeName>_default_relation.
        Please see the Preferences and Environment variables reference in the Rich client interface guide for
        information on configuring these preferences.
        
        
        Use cases:
        Remove Item object from Folder object with contents property name.
        
        Use RemoveChildren operation and provide Folder object as the parent object, Item object as the children object
        and contents as the property name. Also, provide clientId value to identify this remove children operation. The
        RemoveChildren operation will remove Item object from the Folder parent object.
        """
        return cls.execute_soa_method(
            method_name='removeChildren',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'inputData': inputData},
            response_cls=ServiceData,
        )

    @classmethod
    def saveAsObjectsAndRelate(cls, iVecSoaSavAsIn: List[SaveAsIn], iVecSoaRelteInfoIn: List[RelateInfoIn]) -> SaveAsObjectsResponse:
        """
        This operation performs SaveAs on the input target business object and its related objects as new instances.
        Related objects are identifed using deep copy rules. Optionally, this method relates the new object to the
        input target object or to a default folder.
        
        Use cases:
        Use Case 1:     SaveAs without relate
        Client constructs the "SaveAs" dialog for a business object using SaveAs operation descriptor. The information
        returned by that operation allows client to construct the SaveAs dialogs and DeepCopy panels for user input.
        Once the user input is received, client makes subsequent invocation to this operation  to execute SaveAs on the
        object. The method is invoked with "relate" option as false.
        New object is created using values passed in. It is not found under Home / NewStuff folder / anyother parent
        object. The new object stays dangling.
        Use Case 2:     SaveAs and relate to default folder
        Client invokes SaveAs operation as mentioned in use case 1 with "relate" as true but chooses not to specify
        target object or relation. This operation will choose a default folder and choose a default relation to be
        used. The default folder is decided based on the value set for the preference, WsoInsertNoSelectionsPref. When
        the preference value is set as 1 the default folder will be the New Stuff folder of the service user. When the
        preference value is 2 the default folder will be the Home folder of the service user.
        Newly created object is related to the default folder using default relation. For any other value of the
        preference, the relation will not be created.
        Use Case 3:     SaveAs and relate to specified target object using specified relation
        Client invokes SaveAs operation as mentioned in use case 1. The input parameter carrying the relation info has
        the boolean "relate" flag which is true, a valid target object and a property name to which the relation is to
        be created.
        After a successful creation, this operation relates the newly created object to the specified target object
        using specified relation.
        """
        return cls.execute_soa_method(
            method_name='saveAsObjectsAndRelate',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'iVecSoaSavAsIn': iVecSoaSavAsIn, 'iVecSoaRelteInfoIn': iVecSoaRelteInfoIn},
            response_cls=SaveAsObjectsResponse,
        )

    @classmethod
    def generateIdsUsingIDGenerationRules(cls, generateIdsInputs: List[GenerateIdInput]) -> GenerateIdsResponse:
        """
        This operation generates object ids using ID Generation Rules associated with the business object's property.
        Currently only Item and its subtypes are supported. Object ids are generated using information provided in
        createInput.
        This operation should be called in case of a specific requirement where ID Generation is independent of
        creating Objects. (e.g. in case of some CAD applications where ids are created first, used in the system with
        temporary objects which can be saved at the later point of time). In most of the cases
        ''Teamcenter::Soa::Core::_2008_06::createObjects(const std::vector<CreateIn> &input')' handles id Generation
        and object creation. 
        This operation should be invoked when an ID Generation Rule is attached to a Business Object. To identify
        whether the ID Generation Rule is attached to the Business Object, check if 'fnd0IdGenerator' property points
        to the compound Create Descriptor of same Business Object.
        
        For more information on how to configure ID Generation RUles, refer to the Business Modeler IDE Guide. 
        """
        return cls.execute_soa_method(
            method_name='generateIdsUsingIDGenerationRules',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'generateIdsInputs': generateIdsInputs},
            response_cls=GenerateIdsResponse,
        )

    @classmethod
    def getDeepCopyData(cls, deepCopyDataInput: List[DeepCopyDataInput]) -> GetDeepCopyDataResponse:
        """
        This operation returns information required to perform save-as/revise on a POM_object. 
        
        Use cases:
        Client constructs the saveas dialog for a business object using this operation. The information returned by
        this operation allows a client to construct the DeepCopy panels in save-as wizard for user input. Once the user
        input is received, client can make subsequent invocation to the DataManagement.saveAsObjectsAndRelate operation
        to execute SaveAs on the object. 
        """
        return cls.execute_soa_method(
            method_name='getDeepCopyData',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'deepCopyDataInput': deepCopyDataInput},
            response_cls=GetDeepCopyDataResponse,
        )

    @classmethod
    def getPasteRelations2(cls, inputs: List[GetPasteRelationsInputData]) -> GetPasteRelationsResponse2:
        """
        This operation returns the paste relation names for the given parent business objects and the child business
        objects name; the expandable relations and the preferred paste relation are also indicated.
        
        """
        return cls.execute_soa_method(
            method_name='getPasteRelations2',
            library='Core',
            service_date='2014_10',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=GetPasteRelationsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceVis._2018_05.MassiveModelVisualization import GetProductStructureIdResponse2
from typing import List
from tcsoa.gen.Internal.ActiveWorkspaceVis._2015_03.MassiveModelVisualization import ProductStructureIdInput
from tcsoa.utils import TcService


class MassiveModelVisualizationService(TcService):

    @classmethod
    def getStructureIdFromRecipe2(cls, productStructureIdInput: List[ProductStructureIdInput]) -> GetProductStructureIdResponse2:
        """
        This operation retrieves the id that is used to identify a product structure information. The id is a unique
        identifier which gets written into the MMP file as the product Id. The Quicksilver Data server(QDS) maintains
        the product structure information for different product and configurations. Now when the Visualization client
        sends a request to QDS asking information about a product and configuration, the QDS invokes this operation to
        determine the id of the structure which contain information about the requested product and configuration.
        
        Say for example the QDS may have a product structure information with id "UidOfBomIndexAdminData1" that
        contains product structure information of product1 for RevisionRule as "Latest Working" and VariantRule
        "vrule1", "vrule2", "vrule3" and a product structure information with id "UidOfBomIndexAdminData2" that
        contains product structure information of product2 for RevisionRule as "Latest Released" and VariantRule
        "vrule1", "vrule2", "vrule3" and. Now if the Visualization client wants to display the product1 for "Latest
        working" with "vrule2", it contacts the QDS server with this information. The QDS in turn invokes this
        operation with the given information to determine the id of the mmp file, in this case it would return
        "UidOfBomIndexAdminData1" as id in its response.
        
        Use cases:
        The Quicksilver Data server(QDS) is deployed on a LAN closer to the visualization client. The QDS acts as a
        product structure server for the visualization clients. The QDS is configured to periodically communicate to
        the Tcserver and retrieve the indexed product structure information.
        1.    QDS boots and invokes getIndexedProducts and gets the products that are indexed.
        2.     Invokes getStructureFiles operation on each of the indexed product to get the product structure
        information represented in a binary file(*.mmp).
        3.    If the product structure information is returned as a complete file and a set of delta files then the
        client merges the delta file into the complete structure to get the current complete product structure .
        4.    If only delta product structure file is returned then those files are merged with the complete file that
        the client already has.
        5.    A visualization client requests product structure information for a given product and configuration using
        a recipe object. The recipe object could be Awb0ProductContextInfo or 
        VisStructureContext etc. Now QDS invokes getStructureIdFromRecipe to determine which file contains the
        requested product structure information.
        6.    The spatial structure corresponding to the product structure file is then served to the visualization
        client for it to apply massive model visualization algorithm.
        """
        return cls.execute_soa_method(
            method_name='getStructureIdFromRecipe2',
            library='Internal-ActiveWorkspaceVis',
            service_date='2018_05',
            service_name='MassiveModelVisualization',
            params={'productStructureIdInput': productStructureIdInput},
            response_cls=GetProductStructureIdResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2014_11.RequirementsManagement import GetTemplateInput, GetExportTemplateResponse
from typing import List
from tcsoa.utils import TcService
from tcsoa.gen.Internal.AWS2._2014_11.Workflow import WorkflowGraphResponse, WorkflowGraphInput, WorkflowGraphLegendResponse


class WorkflowService(TcService):

    @classmethod
    def getWorkflowGraph(cls, input: List[WorkflowGraphInput]) -> WorkflowGraphResponse:
        """
        This operation returns the latest workflow associated with selected WorkspaceObjects.
        
        Use cases:
        An Active workspace user selects an ItemRevision and wants to see the currently running workflow associated
        with it.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowGraph',
            library='Internal-AWS2',
            service_date='2014_11',
            service_name='Workflow',
            params={'input': input},
            response_cls=WorkflowGraphResponse,
        )

    @classmethod
    def getWorkflowGraphLegend(cls, viewName: str) -> WorkflowGraphLegendResponse:
        """
        This operation returns a localized list of objects and relation type names which are used to represent the
        task(s) and path(s) in a Workflow graph. This list of objects and relation type names is the filter which is
        applied during the graph navigation.
        
        Use cases:
        An Active Workspace user selects an object and opens the Workflow tab. System responds by returning the latest
        workflow associated with the selected object. The task(s) and the path(s) shown in the workflow will have the
        colors, shapes,edges as per the configutation.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowGraphLegend',
            library='Internal-AWS2',
            service_date='2014_11',
            service_name='Workflow',
            params={'viewName': viewName},
            response_cls=WorkflowGraphLegendResponse,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def getExportTemplates(cls, filter: List[GetTemplateInput]) -> GetExportTemplateResponse:
        """
        This operation retrieves export templates from database.These export templates are of type SpecTemplate and
        ExcelTemplate that are used to exporting Teamcenter objects to Office applications using the template option.
        The templates are retrieved based on the input filter.
        """
        return cls.execute_soa_method(
            method_name='getExportTemplates',
            library='Internal-AWS2',
            service_date='2014_11',
            service_name='RequirementsManagement',
            params={'filter': filter},
            response_cls=GetExportTemplateResponse,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2015_07.ScheduleManagement import AssignmentCreateContainer
from tcsoa.gen.ProjectManagement._2012_02.ScheduleManagement import CreatedObjectsContainer
from typing import List
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from datetime import datetime


class ScheduleManagementService(TcService):

    @classmethod
    def shiftSchedule(cls, schedule: Schedule, newDate: datetime, isNewFinishDate: bool, runInBackground: bool) -> ServiceData:
        """
        Shifts the specified Schedule to the new start or finish date. In case of background mode, this operation files
        an asynchronous request to shift the Schedule and releases the client immediately so that the user can perform
        other operation.
        
        Exceptions:
        >If an unhandled error is encountered during the shift schedule operation.
        """
        return cls.execute_soa_method(
            method_name='shiftSchedule',
            library='ProjectManagement',
            service_date='2015_07',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newDate': newDate, 'isNewFinishDate': isNewFinishDate, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def shiftScheduleAsync(cls, schedule: Schedule, newDate: datetime, isNewFinishDate: bool) -> None:
        """
        Shifts the specified Schedule to the new start or finish date. This operation runs asynchronously in its own
        server in the background.
        """
        return cls.execute_soa_method(
            method_name='shiftScheduleAsync',
            library='ProjectManagement',
            service_date='2015_07',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newDate': newDate, 'isNewFinishDate': isNewFinishDate},
            response_cls=None,
        )

    @classmethod
    def assignResources(cls, schedule: Schedule, createAssignments: List[AssignmentCreateContainer]) -> CreatedObjectsContainer:
        """
        Assign Resource
        """
        return cls.execute_soa_method(
            method_name='assignResources',
            library='ProjectManagement',
            service_date='2015_07',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'createAssignments': createAssignments},
            response_cls=CreatedObjectsContainer,
        )



from __future__ import annotations

from tcsoa.gen.ChangeManagement._2020_01.ChangeManagement import DeriveInput
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ChangeManagementService(TcService):

    @classmethod
    def deriveChangeItems(cls, deriveInput: DeriveInput) -> ServiceData:
        """
        This operation derives multiple Change Item objects and carry forward its relations based on the
        propagateRelation flag. When applying deep copy rules, if user overridden deep copy information is provided in
        the input, then old relations are propagated to the new ItemRevision based on the input. If no deep copy
        information is provided in the input, the deep rules in the database will apply.
        
        Use cases:
        &bull;    The requestor (who may be the analyst of the ECR) either derives a new change notice to address the
        approved change request or associates the ECR with an existing ECN. The ECN addresses the implementation
        details of the change. It may address multiple change requests. 
        &bull;    Derive an engineering change request (ECR) from a problem report (PR) to determine a solution for the
        problem.
        &bull;    Derive an engineering change notice (ECN) from an ECR to implement the solution to the problem.
        &bull;    Derive a deviation request from a PR to allow a deviation.
        &bull;    Derive an ECN from multiples ECRs.
        """
        return cls.execute_soa_method(
            method_name='deriveChangeItems',
            library='ChangeManagement',
            service_date='2020_01',
            service_name='ChangeManagement',
            params={'deriveInput': deriveInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from enum import Enum
from typing import Dict, List
from tcsoa.gen.BusinessObjects import ListOfValues
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PropDesc(TcBaseObj):
    """
    The PropDesc struct describes information about the Teamcenter property
    
    :var propName: Name of the property
    :var displayName: Display name of the property
    :var isEnabled: Specifies whether the property is enabled
    :var isModifiable: Specifies whether the property is modifiable
    :var attachedSpecifier: attachedSpecifier
    :var maxLength: maxLength
    :var interdependentProps: interdependentProps
    :var namingPatterns: Naming patterns for property, can be null
    :var defaultValue: Default value for the property
    :var propValueType: Value type for the property, PROP_untyped (0) No Value type PROP_char (1) Value is a single
    character PROP_date (2) Value is a date PROP_double (3) Value is a double PROP_float (4) Value is a float PROP_int
    (5) Value is an integer PROP_logical (6) Value is a logical PROP_short (7) Value is a short PROP_string (8) Value
    is a character string PROP_typed_reference (9) Value is a typed reference PROP_untyped_reference (10) Value is an
    untyped reference PROP_external_reference (11) Value is an external reference PROP_note (12) Value is a note
    PROP_typed_relation (13) Value is a typed relation PROP_untyped_relation (14) Value is an untyped relation
    :var propType: Type for the property PROP_unknown (0) Property type is Unknown PROP_attribute (1)  Based on a POM
    Attribute (int, string, ...) PROP_reference (2)  Based on a POM Reference PROP_relation (3) Based on an
    ImanRelation PROP_compound (4) Based on a property from another Type PROP_runtime (5) Based on a computed value
    :var isDisplayable: isDisplayable
    :var isArray: Specifies whether the property is an array or single value
    :var maxNumElems: Specifies the max number of elements
    :var lov: ListOfValues object attached to the property (if any)
    :var isRequired: Specifies whether the property is required
    """
    propName: str = ''
    displayName: str = ''
    isEnabled: bool = False
    isModifiable: bool = False
    attachedSpecifier: int = 0
    maxLength: int = 0
    interdependentProps: List[str] = ()
    namingPatterns: List[str] = ()
    defaultValue: str = ''
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    maxNumElems: int = 0
    lov: ListOfValues = None
    isRequired: bool = False


@dataclass
class SecCreateDesc(TcBaseObj):
    """
    Secondary Create Descriptor definition. This is the secondary create descriptor data type for secondary objects
    that get created during object creation. For example, ItemRevision and Item Master are the secondary objects that
    get created during creation of Item. Note that this is a recursive data structure which can itself point to other
    Secondary Create Descriptors (ItemRevision pointing to ItemRevision Master)
    
    :var businessObjectTypeName: Business Object Type Name of the secondary object
    :var isRequired: Boolean field indicating if creation of secondary object is required (non optional) or not
    :var isArray: true indicates the secondary descriptor is part of an array of secondary descriptors (The relation or
    reference property on the primary create descriptor could potentially point to several secondary descriptors i.e
    Create several secondary objects for the same reference or relation property. This is the usecase for which the
    field will be true for each Secondary Descriptor in the array)
    :var compoundingCtxt: enum indicating if the secondary object is a relation (in which case value is
    'CompoundRelation') or a regular business object (in which case value is 'CompoundObject')
    :var propDescs: List of Property Descriptors for the Secondary Object
    :var secondaryCreateDescs: Map '(string, Teamcenter::Soa::Core::_2008_06::PropDescriptor::SecCreateDesc') of the
    secondary descriptor objects. For example, an ItemRevision is the Secondary CreateDescriptor for Item. The
    ItemRevision Secondary Create Descriptor itself will contain SecCreateDesc objects for ItemRevision Master which is
    also created when the ItemRevision is created. The map contains the reference property or relation property name on
    the Parent Secondary Business Object as the key and the 'SecCreateDesc' as the value. If there are no secondary
    objects to be created, the map will be empty (For example, the secondary create descriptor for Item Master  which
    is created when Item is created has an empty map)
    """
    businessObjectTypeName: str = ''
    isRequired: bool = False
    isArray: bool = False
    compoundingCtxt: CompoundingContext = None
    propDescs: List[PropDesc] = ()
    secondaryCreateDescs: SecCreateDescMap = None


@dataclass
class CreateDesc(TcBaseObj):
    """
    Create Descriptor definition
    
    :var businessObjectTypeName: Business Object Type Name
    :var propDescs: Vector of Property Descriptors
    :var secondaryCreateDescs: Map of property name to secondary object's create descriptors
    """
    businessObjectTypeName: str = ''
    propDescs: List[PropDesc] = ()
    secondaryCreateDescs: SecCreateDescMap = None


@dataclass
class CreateDescResponse(TcBaseObj):
    """
    Structure containing Create Descriptor information representing the metadata about the properties relevant to a
    Create Operation
    
    :var createDescs: List of 'Teamcenter::Soa::Core::_2008_06::PropDescriptor::CreateDesc'  objects. Each element in
    the list is a Create Descriptor for a business object which contains metadata information about the properties
    necessary to create the business object i.e, property is mandatory, property is visible, etc. It is a recursive
    data structure which may point to secondary 'CreateDesc' objects e.g Item 'CreateDesc' contains references to
    ItemRevision and Item Master 'CreateDesc' objects
    :var srvData: Service data containing partial errors. If there is any error in retrieving the Create Descriptor for
    any Business Object in the input vector, it is returned as part of the error stack. The service data also contains
    LOV objects for the properties in the plain objects vector in the service data
    """
    createDescs: List[CreateDesc] = ()
    srvData: ServiceData = None


class CompoundingContext(Enum):
    """
    Enum for compounding context. Used to indicate if secondary (compounded) object is a compound object (value will be
    'CompoundObject') or compound relation (value will be 'CompoundRelation'). When the context is 'CompoundRelation',
    it is usually for the roperties on Relation use case when the Relation creation descriptor itself may have
    properties configured for creation 
    NOTE: 'CompoundRelation' case is not supported at this time
    """
    CompoundObject = 'CompoundObject'
    CompoundRelation = 'CompoundRelation'


"""
SecCreateDescMap
"""
SecCreateDescMap = Dict[str, List[SecCreateDesc]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    a generic structure to capture additional information.
    - intMap    A map of string to a list of integers. 
    - dblMap    A map of string to a list of doubles. 
    - strMap    A map of string to a list of strings. 
    - objMap    A map of string to a list of business objects.
    - dateMap    A map of string to a list of dates.
    
    
    
    :var intMap: A map (string/list of integers) of generic key to integer values.
    :var dblMap: A map (string/list of doubles) of generic key to double values.
    :var strMap: A map (string/list of strings) of generic key to string values.
    :var objMap: A map (string/list of BusinessObjects) of generic key to  BusinessObject values.
    :var dateMap: A map (string/list of dates) of generic key to date values.
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringtoStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


@dataclass
class CreateOrUpdatePreviousEffResp(TcBaseObj):
    """
    CreateOrUpdatePreviousEffResp structure contains a list of AdditionalInfo elements, the size of which matches the
    input element list. It also includes the standard ServiceData object.
    
    :var info: A list of AdditionalInfo structures. The created or updated ConfigurationContext object representing the
    previous effectivity will appear under the objMap key "Effectivity".
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    info: List[AdditionalInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ConnectChangeNoticeToContextInElem(TcBaseObj):
    """
    ConnectChangeNoticeToContextInElem structure contains the details of a single element necessary for associating a
    changeObject ( currently, must be ChangeNoticeRevision) with the optional container ( currently, must be BOMWindow).
    
    :var changeObject: The ChangeNoticeRevision which is to be used to associate with the context. Currently, only
    BOMWindow object is supported for context.
    :var context: A context to which the changeNoticeRevision is to be associated. Currently, if specified must be
    BOMWindow object.
    :var shareMode: The mode to be used for determining how the revisions in the solution folder of the
    ChangeNoticeRevision are to be released. If this optional argument is not empty, it must be one of "Share", "None",
    "Clone", "ShareEffectivityOnly". 
    - Share: share the release status on revisions with the "preferred" release status On ChangeNoticeRevision that is
    a subset of the values given by preference "MEMCN_release_statues".
    - None: no release status on revisions.
    - Clone: new release statuses on revisions cloned from "preferred" release status.
    - ShareEffectivityOnly: new release statues on revisions with same type as the "preferred" release status on
    ChangeNoticeRevision, but with shared effectivity. If specified, will set the "cm0ChangeNoticeRevShareMode" runtime
    property on the BOMWindow.
    
    
    :var info: Currently, not used.
    """
    changeObject: BusinessObject = None
    context: BusinessObject = None
    shareMode: str = ''
    info: AdditionalInfo = None


@dataclass
class PreviousEffectivity(TcBaseObj):
    """
    PreviousEffectivity structure contains the details of a single element necessary for creating or updated a
    ConfigurationContext and associating with the ChangeNoticeRevision.
    
    :var changeObject: The ChangeNoticeRevision which is to be used to associate with the ConfigurationContext created
    representing previous effectivity.
    :var endItem: The Item or ItemRevision that is the endItem of the ConfigurationContext to be associated with
    ChangeNoticeRevision.
    :var unit: The text representing the unit of previous effectivity.
    :var date: The DateTime object representing the date of the previous effectivity.
    :var info: Currently, not used
    """
    changeObject: BusinessObject = None
    endItem: BusinessObject = None
    unit: str = ''
    date: datetime = None
    info: AdditionalInfo = None


@dataclass
class ConnectChangeNoticeToContextResp(TcBaseObj):
    """
    ConnectChangeNoticeToContextResp structure contains a list of AdditionalInfo elements, the size of which matches
    the input element list. It also includes the standard ServiceData object. The created ConfigurationContext object
    will be returned as the value of objMap key "Effectivity".
    
    :var info: Currently, not used. 
    :var serviceData: Service data capturing partial errors using the input array index as client id and any updated
    BOMWindow objects.
    """
    info: AdditionalInfo = None
    serviceData: ServiceData = None


@dataclass
class UpdateChangeNoticeRelationsIn(TcBaseObj):
    """
    UpdateChangeNoticeRelationsIn structure contains the details necessary for managing the secondary ItemRevision
    objects related to the ChangeNoticeRevision, which is associated with a BOMWindow.
    
    :var selectedLines: The selected BOMLine objects whose ItemRevision objects will be added to or removed from the
    ChangeNoticeRevision relations.
    :var action: Indicate if this is adding or removing ItemRevision objects. Current valid values are: "Add" and
    "Remove".
    :var isRecursive: Indicate if the children of the selectedLines will be considered as well. If true, the children
    of the selectedLines will be traversed based on a closure rule specified by preference
    MEMCN_solution_object_collection_rule. Otherwise, only the selectedLines will be considered by the closure rule. 
    :var additionalInfo: An AdditionalInfo structure. Currently, not used.
    """
    selectedLines: List[BusinessObject] = ()
    action: str = ''
    isRecursive: bool = False
    additionalInfo: AdditionalInfo = None


@dataclass
class UpdateChangeNoticeRelationsResp(TcBaseObj):
    """
    UpdateChangeNoticeRelationsResp structure contains a list of AdditionalInfo elements, the size of which matches the
    input element list. It also includes the standard ServiceData object. 
    
    :var info: A list of AdditionalInfo structures. Currently not used.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    info: List[AdditionalInfo] = ()
    serviceData: ServiceData = None


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to vector of strings.
"""
StringtoStrVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Vendormanagement._2007_06.VendorManagement import CreateVendorPartsResponse
from typing import List
from tcsoa.utils import TcService
from tcsoa.gen.Vendormanagement._2016_09.VendorManagement import VendorPartProperties2


class VendorManagementService(TcService):

    @classmethod
    def createVendorParts(cls, properties: List[VendorPartProperties2]) -> CreateVendorPartsResponse:
        """
        This operation creates a set of ManufacturerPart objects of given type based on input data.This operation is
        applicable for bulk creation of Vendor Part objects.
        
        Use cases:
        Create Vendor Parts in bulk,when Vendor is already created and CompanyLocation can be available with Vendor.
        """
        return cls.execute_soa_method(
            method_name='createVendorParts',
            library='Vendormanagement',
            service_date='2016_09',
            service_name='VendorManagement',
            params={'properties': properties},
            response_cls=CreateVendorPartsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, ImanFile
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class DatashareManagerReplaceInfo(TcBaseObj):
    """
    Information about a file to be replaced asynchronously via Datashare Manager.
    
    :var dataset: The Dataset object to which the reference ImanFile object(s) are to be uploaded to the Teamcenter
    volume.
    :var imanFile: The ImanFile currently referenced by the Dataset.  This named reference is intended to be replaced
    by the operation.
    :var namedReferenceName: The named reference relation to the ImanFile object file.
    :var absoluteFilePath: The absolute file path of the file being uploaded on the client host, or an empty string if
    the end user should select the file to be uploaded by using the Data Share Manager user interface.
    """
    dataset: Dataset = None
    imanFile: ImanFile = None
    namedReferenceName: str = ''
    absoluteFilePath: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Visualization._2008_06.StructureManagement import ExpandPSFromOccurrenceListResponse, CreateBOMsFromRecipesResponse, CreateVisSCsFromBOMsResponse, CreateBOMsFromRecipesInfo, ExpandPSFromOccurrenceListInfo, CreateVisSCsFromBOMsInfo, AreRecipesMergableResponse, ExpandPSFromOccurrenceListPref, AreRecipesMergableInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Visualization._2008_06.DataManagement import CreateTwoDSnapshotResponse, GetLatestFileReadTicketsInfo, SessionModelUpdateResponse, MarkupInput, UpdateSesssionModel, UpdateSessionModelWithSessionInput, MarkupUpdateResponse, ServerInfo, VVITicketsResponse, MetaDataStampTicketsResponse, SessionInfo, MarkupResponse, SessionModelInput, UserAgentDataInfo, IdInfo, MarkupUpdateInput, GetLatestFileReadTicketsResponse, TwoDSnapshotInput, SaveSessionResponse, SessionModelResponse, UpdateTwoDSnapshotInput, UpdateTwoDSnapshotResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def saveSession(cls, inputs: List[UpdateSessionModelWithSessionInput]) -> SaveSessionResponse:
        """
        This operation is responsible for committing the session file (of extension .vf) to the Vis_Session Dataset.
        This is done after the Vis_Session Dataset is created via 'createSessionModel' visualization operation or
        updated via the 'updateSessionModel' visualization operation (See Dependency section).  Creating or updating of
        the Vis_Session Dataset involves committing miscellaneous files like markups, images, other dependent files
        etc. and necessary relations to the Teamcenter objects as per the data model. The existence of all the relevant
        Teamcenter managed objects referenced by the session are recorded in the Teamcenter visualization session file
        and committed to the Vis_Session Dataset via 'saveSession' operation. Associated Dataset and IMANFile objects
        are then accessible to other authorized clients throughout the PLM system.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        
        Use cases:
        The mechanism for a client application to save session contains several steps.
        
        1.    Client application invokes the 'FileManagementService::getWriteTickets' operation to obtain an FMS write
        ticket for each file / IMANFile object that will be stored in the markup Dataset. 
        
        2.    Use a FMS operation to upload the files to the PLM volume (see the Dependencies section below).  The
        client application is responsible for making sure that the FMS operation used to upload the files indicates
        that the file uploads were successful. If a file upload were to fail, then the file should not be committed. 
        This should effectively prevent the construction of database infrastructure referencing files not present in
        the volume.  Check the documentation for each operation to determine how to react to upload failures.  Some
        upload operations are defined in such a way that a single result code is returned to the caller, or a single
        exception thrown, to indicate that the bulk file upload was entirely unsuccessful.  These operations typically
        roll back all the successfully uploaded files before returning to the caller.  Some operations may accept a
        flag indicating whether uploads should continue when an error is encountered.  Other operations return a
        separate result code for each file, indicating which files were successfully uploaded to a PLM volume and which
        uploads failed.
        
        It is the responsibility of the client application to either commit or rollback all successfully uploaded
        files.  Files that failed to upload need not be rolled back.
        
        Client responsibilities if upload of files to the volume fails:
        a.    Client applications must not invoke the 'saveSession' visualization operation on files that did not
        successfully upload to the PLM volume.  This causes file access errors downstream, when clients attempt to
        reference (or download) the file data, which will be missing from the PLM volume.
        b.    In the event of a partial upload success (some files uploaded successfully, but others did not), it is
        the responsibility of the client application to determine whether it is acceptable in context to commit the
        successfully uploaded files to the PLM system.
        - If not, then there is a companion FMS operation to roll back the successfully uploaded files (see
        Dependencies section).  The rollback method removes the files from the PLM volume.
        - If so, then it is the responsibility of the client application to avoid including information corresponding
        to the files that did not successfully upload to the PLM volume in the input list passed to the 'saveSession'
        visualization operation.
        
        
        
        3.    The client application then invokes the 'createSessionModel' or 'updateSessionModel' visualization
        operations on the successfully uploaded files. 
        This step creates or updates visualization supported session Dataset of types Vis_Session and commits the
        uploaded files to the Dataset.  In addition, these operations also associate the session Dataset to input
        Teamcenter objects with relevant relations as per the data model. Associated Dataset and IMANFile objects are
        then accessible to other authorized clients through the PLM system. If there are any failures during this
        operation, the client is responsible for rolling back the associated file from the volume through the FMS
        operation.  Check the documentation for each API to determine how to react to cleanup activity.
        
        Note:  In cases where the session is created against BOMView or BOMViewRevision or Teamcenter runtime objects
        like BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the responsibility
        of the client to create VisStructureContext object and provide VisStructureContext as an input to
        'createSessionModel' or 'updateSessionModel' operation.
        
        4.    After creating or updating the Vis_Session Dataset via 'createSessionModel' or 'updateSessionModel'
        respectively, the information of all the relevant Teamcenter managed objects in the session is noted in the
        session file (.vf). An FMS write ticket is then obtained for the session file and uploaded via FMS operation to
        Teamcenter volume.
        
        5.    Use 'saveSession' visualization operation to commit the session file to the Vis_Session Dataset.
        
        NOTE: It is the responsibility of the client to do cleanup as appropriate according to failures types.
        
        This operation is used in conjunction with other 'FileManagementService' service operations, 'Visualization'
        service operations and the FMS.  Please consult the documentation for each of these available operations for
        details on the requirements, usage, and environments in which they should be used.
        
        These dependencies include:
        
        1.    FMS operation for obtaining an FMS write ticket.
        
        'FileManagementService::getWriteTickets'    This operation obtains FMS write tickets for IMANFile objects.
        
        2.    FMS operations for uploading and rolling back the files to/from the PLM volume.
        
        Library                        Binding
        
        FCCClientProxy.lib                C
        
        'FCC_UploadFileToPLM'        Uploads a file to a PLM volume.
        'FCC_UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FCC_RollbackFile'                Removes a file from a PLM volume.
        'FCC_RollbackFiles'                Removes multiple files from a PLM volume.
        
        fccjavaclientproxy.jar            Java
        
        'FileCacheProxy.UploadFileToPLM'        Uploads a file to a PLM volume.
        'FileCacheProxy.UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FileCacheProxy.RollbackFile'                Removes a file from a PLM volume.
        'FileCacheProxy.RollbackFiles'            Removes multiple files from a PLM volume.
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll        C
        
        'FSC_Upload'                    Uploads a file to a PLM volume.
        'FSC_UploadMultiple'        Uploads multiple files to a PLM volume.
        'FSC_Rollback'                Removes a file from a PLM volume.
        'FSC_RollbackMultiple'        Removes multiple files from a PLM volume.
        
        fscjavaclientproxy.jar            Java
        
        'CommonsFSCWholeFileIOImpl.upload'        Uploads one or more files to a PLM volume.
        'CommonsFSCWholeFileIOImpl.rollback'        Removes one or more files from a PLM volume.
        
        
        3.    Visualization operations for creating the VisStructureContext when the markups are created against
        Teamcenter objects like BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines from Structure
        Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs (const std::vector< Teamcenter::Soa::Internal::Visualization::_2008_06::
        StructureManagement::CreateVisSCsFromBOMsInfo > & info)'; 
        
        4.    Visualization operations for creating or updating the session Dataset, associating the Vis_Session
        Dataset with the input Teamcenter objects and committing the files to the session Dataset.
        
        NOTE:  'createSessionModel' / 'updateSessionModel' operation should be followed by 'saveSession' operation.
        """
        return cls.execute_soa_method(
            method_name='saveSession',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveSessionResponse,
        )

    @classmethod
    def updateMarkup(cls, inputs: List[MarkupUpdateInput]) -> MarkupUpdateResponse:
        """
        This operation updates the markup Datasets of types DirectModel3DMarkup, Markup or DirectModelMarkup and
        commits the uploaded files to the Dataset.  In addition, this operation may associate the markup Dataset to an
        input Teamcenter object (e.g. VisStructureContext) with relevant relation as per the data model.  Associated
        Dataset and ImanFile objects are then accessible to other authorized clients throughout the PLM system.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        
        Use cases:
        The mechanism for a client application to update markups requires several steps.
        
        1.    Client application invokes the 'FileManagementService::getWriteTickets' operation to obtain an FMS write
        ticket for each file / ImanFile object that will be stored in the markup Dataset. 
        
        2.    Use a FMS operation to upload the files to the PLM volume (see the Dependencies section below).  The
        client application is responsible for making sure that the FMS operation used to upload the files indicates
        that the file uploads were successful. If a file upload were to fail, then the file should not be committed. 
        This should effectively prevent the construction of database infrastructure referencing files not present in
        the volume.  Check the documentation for each operation to determine how to react to upload failures.  Some
        upload operations are defined in such a way that a single result code is returned to the caller, or a single
        exception thrown, to indicate that the bulk file upload was entirely unsuccessful.  These operations typically
        roll back all the successfully uploaded files before returning to the caller.  Some operations may accept a
        flag indicating whether uploads should continue when an error is encountered.  Other operations return a
        separate result code for each file, indicating which files were successfully uploaded to a PLM volume and which
        uploads failed.
        
        It is the responsibility of the client application to either commit or rollback all successfully uploaded
        files.  Files that failed to upload need not be rolled back.
        
        Client responsibilities if upload of files to the volume fails:
        a.    Client applications must not invoke the 'updateMarkup' visualization operation on files that did not
        successfully upload to the PLM volume.  This causes file access errors downstream, when clients attempt to
        reference (or download) the file data, which will be missing from the PLM volume.
        b.    In the event of a partial upload success (some files uploaded successfully, but others did not), it is
        the responsibility of the client application to determine whether it is acceptable in context to commit the
        successfully uploaded files to the PLM system.
        - If not, then there is a companion FMS operation to roll back the successfully uploaded files (see
        Dependencies section).  The rollback method removes the files from the PLM volume.
        - If so, then it is the responsibility of the client application to avoid including information corresponding
        to the files that did not successfully upload to the PLM volume in the input list passed to the 'updateMarkup'
        visualization operation.
        
        
        
        3.    The client application then invokes the 'updateMarkup' visualization operation on the successfully
        uploaded files.   This step updates the visualization supported markup Dataset of types DirectModel3DMarkup,
        Markup, DirectModelMarkup and commits the uploaded files to the Dataset.  In addition, this operation may
        associate the updated markup Dataset with input Teamcenter object like VisStructureContext with relevant
        relation as per the data model. Associated Dataset and IMANFile objects are then accessible to other authorized
        clients through the PLM system. If there are any failures during this operation, client is responsible for
        rolling back the associated file from the volume through FMS operations.  Check the documentation for each
        operation to determine how to react to cleanup activity.
        
        Note:  In case markups are created against BOMView or BOMViewRevision or Teamcenter runtime objects like
        BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the responsibility of the
        client to create VisStructureContext object and provide VisStructureContext as an input to the operation.
        
        Use Case References:
        This operation is used in conjunction with other FileManagementService service operations, Visualization
        service operations and the FMS.  Please consult the documentation for each of these available operations for
        details on the requirements, usage, and environments in which they should be used.  
        
        These dependencies include:
        
        1.    FMS operation for obtaining a FMS write ticket:
        'FileManagementService::getWriteTickets'    This operation obtains FMS write tickets for IMANFile objects.
        
        2.    FMS methods for uploading and rolling back the files to/from the PLM volume.
        
        Library                        Binding
        
        FCCClientProxy.lib                C
        
        'FCC_UploadFileToPLM'        Uploads a file to a PLM volume.
        'FCC_UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FCC_RollbackFile'                Removes a file from a PLM volume.
        'FCC_RollbackFiles'                Removes multiple files from a PLM volume.
        
        fccjavaclientproxy.jar            Java
        
        'FileCacheProxy.UploadFileToPLM'        Uploads a file to a PLM volume.
        'FileCacheProxy.UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FileCacheProxy.RollbackFile'                Removes a file from a PLM volume.
        'FileCacheProxy.RollbackFiles'            Removes multiple files from a PLM volume.
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll        C
        
        'FSC_Upload'                    Uploads a file to a PLM volume.
        'FSC_UploadMultiple'        Uploads multiple files to a PLM volume.
        'FSC_Rollback'                Removes a file from a PLM volume.
        'FSC_RollbackMultiple'        Removes multiple files from a PLM volume.
        
        fscjavaclientproxy.jar            Java
        
        'CommonsFSCWholeFileIOImpl.upload'        Uploads one or more files to a PLM volume.
        'CommonsFSCWholeFileIOImpl.rollback'        Removes one or more files from a PLM volume.
        
        3.    Visualization operations for creating the VisStructureContext when the markups are created against
        Teamcenter objects like BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines from Structure
        Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs (const std::vector< Teamcenter::Soa::Internal::Visualization::_2008_06::
        StructureManagement::CreateVisSCsFromBOMsInfo > & info); '
        
        4.    Visualization operations for creating the markup Dataset, associating the created markup Dataset and the
        input Teamcenter objects and committing the files to the markup Dataset.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::MarkupResponse createMarkup ( const
        std::vector<Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::MarkupInput >& inputs ) ;'
        """
        return cls.execute_soa_method(
            method_name='updateMarkup',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=MarkupUpdateResponse,
        )

    @classmethod
    def updateSessionModel(cls, inputs: List[UpdateSesssionModel]) -> SessionModelUpdateResponse:
        """
        This operation updates the Vis_Session Dataset and commits the uploaded files to the Dataset. Updating of
        Vis_Session Dataset involves committing miscellaneous files like markups, images, dependent files etc. and
        attaching necessary relations to the Teamcenter objects as per the data model. This operation must be followed
        by a call to the 'saveSession' operation. The information of all the relevant Teamcenter managed objects in the
        session is noted in the Teamcenter visualization session file and committed to the Vis_Session Dataset via the
        'saveSession' operation. Associated Dataset and IMANFile objects are then accessible to other authorized
        clients through the PLM system.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        
        Use cases:
        The mechanism for a client application to update session contains several steps.
        
        1.    Client application invokes the' FileManagementService::getWriteTickets' operation to obtain an FMS write
        ticket for each file / IMANFile object that will be stored in the markup Dataset. 
        
        2.    Use a FMS operation to upload the files to the PLM volume (see the Dependencies section below).  The
        client application is responsible for making sure that the FMS operation used to upload the files indicates
        that the file uploads were successful. If a file upload were to fail, then the file should not be committed. 
        This should effectively prevent the construction of database infrastructure referencing files not present in
        the volume.  Check the documentation for each operation to determine how to react to upload failures.  Some
        upload operations are defined in such a way that a single result code is returned to the caller, or a single
        exception thrown, to indicate that the bulk file upload was entirely unsuccessful.  These operations typically
        roll back all the successfully uploaded files before returning to the caller.  Some operations may accept a
        flag indicating whether uploads should continue when an error is encountered.  Other operations return a
        separate result code for each file, indicating which files were successfully uploaded to a PLM volume and which
        uploads failed.
        
        It is the responsibility of the client application to either commit or rollback all successfully uploaded
        files.  Files that failed to upload need not be rolled back.
        
        Client responsibilities if upload of files to the volume fails:
        a.    Client applications must not invoke the 'updateSessionModel' visualization operation on files that did
        not successfully upload to the PLM volume.  This causes file access errors downstream, when clients attempt to
        reference (or download) the file data, which will be missing from the PLM volume.
        b.    In the event of a partial upload success (some files uploaded successfully, but others did not), it is
        the responsibility of the client application to determine whether it is acceptable in context to commit the
        successfully uploaded files to the PLM system.
        - If not, then there is a companion FMS operation to roll back the successfully uploaded files.  The rollback
        method removes the files from the PLM volume.
        - If so, then it is the responsibility of the client application to avoid including information corresponding
        to the files that did not successfully upload to the PLM volume in the input list passed to the
        'updateSessionModel' visualization operation.
        
        
        
        3.    The client application then invokes the 'updateSessionModel' visualization operation on the successfully
        uploaded files. 
        This step updates the session Dataset of type Vis_Session and commits the uploaded files to the Dataset.  In
        addition, this operation also associates the session Dataset to the input Teamcenter objects with relevant
        relations as per the data model. Associated Dataset and ImanFile objects are then accessible to other
        authorized clients through the PLM system. If there are any failures during this operation, client is
        responsible for rolling back the associated file from the volume through FMS operation.  Check the
        documentation for each operation to determine how to react to cleanup activity.
        
        Note:  In case the session is created against BOMView or BOMViewRevision or Teamcenter runtime objects like
        BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the responsibility of the
        client to create VisStructureContext object and provide VisStructureContext as an input to the
        'updateSessionModel' operation.
        
        4.    After updating the Vis_Session Dataset via 'updateSessionModel', the information for all the relevant
        Teamcenter managed objects in the session is noted in the session file (.vf). An FMS write ticket is then
        obtained for the session file and the file uploaded via FMS operation to Teamcenter volume.
        
        5.    Use 'saveSession' visualization operation to commit the session file to the Vis_Session Dataset.
        
        NOTE: It is the responsibility of the client to do cleanup as appropriate according to failures types.
        
        Use Case References:
        
        This operation is used in conjunction with other FileManagementService service operations, Visualization
        service operations and the FMS.  Please consult the documentation for each of these available operations for
        details on the requirements, usage, and environments in which they should be used.  These dependencies include:
        
        1.    FMS operation for obtaining an FMS write ticket.
        'FileManagementService::getWriteTickets'        This operation obtains FMS write tickets for IMANFile objects 
        
        2.    FMS operations for uploading and rolling back the files to/from the PLM volume.
        
        Library                        Binding
        
        FCCClientProxy.lib                C
        
        'FCC_UploadFileToPLM'        Uploads a file to a PLM volume.
        'FCC_UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FCC_RollbackFile'                Removes a file from a PLM volume.
        'FCC_RollbackFiles'                Removes multiple files from a PLM volume.
        
        fccjavaclientproxy.jar            Java
        
        'FileCacheProxy.UploadFileToPLM'        Uploads a file to a PLM volume.
        'FileCacheProxy.UploadFilesToPLM    '    Uploads multiple files to a PLM volume.
        'FileCacheProxy.RollbackFile'                Removes a file from a PLM volume.
        'FileCacheProxy.RollbackFiles'            Removes multiple files from a PLM volume.
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll        C
        
        'FSC_Upload'                    Uploads a file to a PLM volume.
        'FSC_UploadMultiple'        Uploads multiple files to a PLM volume.
        'FSC_Rollback'                Removes a file from a PLM volume.
        'FSC_RollbackMultiple'        Removes multiple files from a PLM volume.
        
        fscjavaclientproxy.jar            Java
        
        'CommonsFSCWholeFileIOImpl.upload'        Uploads one or more files to a PLM volume.
        'CommonsFSCWholeFileIOImpl.rollback'        Removes one or more files from a PLM volume.
        
        3.    'Visualization' operations for creating the VisStructureContext when the markups are created against
        Teamcenter objects like BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines from Structure
        Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs (const std::vector< Teamcenter::Soa::Internal::Visualization::_2008_06::
        StructureManagement::CreateVisSCsFromBOMsInfo > & info); '
        
        4.    'Visualization' operations for creating or updating the session Dataset, associating the Vis_Session
        Dataset with the input Teamcenter objects and committing the files to the session Dataset.
        
        NOTE:  'createSessionModel' / 'updateSessionModel' operation should be followed by 'saveSession' operation.
        """
        return cls.execute_soa_method(
            method_name='updateSessionModel',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SessionModelUpdateResponse,
        )

    @classmethod
    def updateTwoDSnaphot(cls, input: List[UpdateTwoDSnapshotInput]) -> UpdateTwoDSnapshotResponse:
        """
        A 2D snapshot dataset (Vis_snapshot_2D_View_Data) is an object in Teamcenter used to capture the state of a 2D
        scene including markups and viewport information.  This object is used for manufacturing planning and issue
        capture use cases.  The dataset and associated data model is designed to support change management, and updates
        to the base image without significant rework to the previously captured snapshots.  That is, the snapshots can
        be easily re-used and updated.  The data model consists of a dataset storing several files (markup layer CGM
        files, viewport xml file, thumbnail image, and asset image), a relation to the ItemRevision managing the base
        image, and a form containing attributes that can be used to search for snapshots based on certain attribute
        criteria. 
        This service is focused on updating a 2D snapshot dataset that has been previously created with the
        createTwoDSnapshot service.  Several of the input structures are shared between create and update services. 
        This service updates a list of 2D snapshot Datasets with the named references provided.
        
        Use cases:
        User updates a snapshot of a particular view of a 2D image with markups to illustrate an area of interest. 
        This snapshot dataset is updated in Teamcenter and later re-opened, recreating the view.
            1.    User opens a 2D snapshot dataset that has been previously created.  This can be done by sending the
        dataset to the MPP application or Lifecycle viewer, or by finding the dataset in the 2D snapshot gallery of the
        2D viewer embedded in the Manufacturing Process Planner (MPP) application
            2.    The system recreates the 2D scene represented by the 2D snapshot by opening the base image and
        applying the viewfile information and markups
            3.    The user updates the 2D scene by changing it in some way.  Typical changes are changing the view
        port, editing the markup layers, or editing the form data
            4.    User updates the 2D snapshot using the appropriate user action.
            5.    The system calls the updateTwoDSnapshot service and provides the new information including new
        dataset information (e.g. type, name, description), and new named reference information (e.g. new markup layer
        files, new view file, etc).  The dataset is updated with the appropriate data model by this service.
        
        When the Vis_snapshot_2D_View_Data dataset is updated, a new version of the dataset is created with the new and
        updated information.   
        """
        return cls.execute_soa_method(
            method_name='updateTwoDSnaphot',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=UpdateTwoDSnapshotResponse,
        )

    @classmethod
    def createLaunchFile(cls, idInfos: List[IdInfo], serverInfo: ServerInfo, userDataAgentInfo: UserAgentDataInfo, sessionInfo: SessionInfo) -> VVITicketsResponse:
        """
        This operation generates a VVI file which is used to launch integrated standalone visualization or Lifecycle
        Viewer with selected objects from Teamcenter and preserve a two way communication link between the viewer and
        the server.  These files are not intended to be permanent (i.e. transient) and should be generated with each
        launch.  For example, the VVI format is not guaranteed to be supported if the server or viewer is updated.  VFZ
        files are used if more than one object is launched at a time.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes) which is used to download the VVI file from the transient file volume to the client.  This
        operation generates the launch file (VFZ or VVI), stores it in the FMS transient volume, and returns the FMS
        ticket.  The client that initiated this operation is responsible for downloading the transient file (VVI or
        VFZ) with the transient ticket from transient volume to a local file system. Transient (VVI or VFZ) file is
        then consumed by the visualization client.  The viewer will then establish the server connection and load the
        object(s) specified in the VVI file.  Launch on multiple objects will generate a VFZ file (zip of all the VVI
        files) and transient ticket of VFZ file would be sent to client. 
        
        This service supports launch on one or more Teamcenter persistent objects such as Dataset, Item, ItemRevision,
        BOMViewRevision, and BOMView. It also supports launch of configured structure containing BOMLines from
        Structure Manager or BOPLines from Manufacturing Process Planner, but in this case the caller must first create
        a VisStructureContext object (See Dependency section for operation to use).  Valid launch object types and
        order preference can be configured with the Teamcenter Preferences VMU_Datasets, VMU_FileSearchOrder and
        VMU_RelationSearchOrder.
        
        
        Use cases:
        This operation supports the mechanism of visualizing Teamcenter specific objects in Teamcenter Visualization
        client. There are several steps to support this mechanism.
        
        1.    The client application that initiates the launch will provide:
        
        A vector of IdInfo objects that contains one or more pieces of information about Teamcenter objects that needs
        to be visualized in the viewer (e.g., If Dataset is launched, then information about its Item, ItemRevision and
        type of operation, including any additional information can be provided). Note: In case launch of Teamcenter
        runtime objects like BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the
        responsibility of the client to create VisStructureContext object and provide VisStructureContext as the object
        to be launched (See Dependency section for operation to use).
        SessionInfo object contains session relevant information for Teamcenter Visualization to connect to the session
        (e.g., session discriminator and any other additional session relevant key value pair)
        ServerInfo object contains server information for Teamcenter Visualization to connect to the server. (e.g.,
        protocol, server URL, connection mode of the server and any other additional server relevant key value pair)
        UserAgentDataInfo object contains client application information who initiated the launch. (e.g., application
        name, application version, and any other additional client application relevant key value pair).
        
        
        2.    After gathering the necessary information as listed in step 1, client application then invokes the
        DataManagementService::createLaunchFile operation to obtain an FMS read ticket for the launch file (VVI or
        VFZ), that has relevant information for visualizing Teamcenter persistent or runtime objects.
        
        See the Dependencies section below for details.
        
        3.    Use a File Management System (FMS) Application Programmatic Interface (API) to download the transient
        file (VVI or VFZ) from transient volume.
        
        Check the documentation for each API to determine how to react to download failures.
        
        Use Case References:
        This operation is used in conjunction with other FileManagementService service operations, Visualization
        service operations and the FMS. Please consult the documentation for each of these available operations for
        details on the requirements, usage, and environments in which they should be used. 
        
        These references include:
        
        1.    FMS methods for registering the transient file tickets prior to downloading the transient file.
        
        Library                    Binding
        FCCClientProxy.lib            C            
        
        FCC_RegisterTicket        Decrypts the ticket, registers the File with client cache, and returns File UID that
        may or may not correspond to the UID of the ticketed file.
        FCC_RegisterTickets    Decrypts multiple tickets, registers the Files with client cache, and returns File UIDs
        that may or may not correspond to the UID of the ticketed file.
        
        
        fccjavaclientproxy.jar        Java
        
        FileCacheProxy.RegisterTicket        Decrypts the ticket, registers the File with client cache, and returns
        File UID that may or may not correspond to the UID of the ticketed file.
        FileCacheProxy. RegisterTickets        Decrypts multiple tickets, registers the Files with client cache, and
        returns File UIDs that may or may not correspond to the UID of the ticketed file.
        
        
        2.    FMS methods for downloading the single transient file to a local file system from Teamcenter Transient
        volume.
        
        Library                    Binding
        FCCClientProxy.lib            C
        
        FCC_DownloadFileToLocation        Downloads a file from a Transient volume into a local filename specified by
        the caller.
        FCC_DownloadFilesToLocation    Downloads multiple files from a Transient volume into local filenames specified
        by the caller.
        
        
        fccjavaclientproxy.jar        Java
        
        FileCacheProxy.DownloadFileToLocation    Downloads a file from a Transient volume into a local filename
        specified by the caller.
        FileCacheProxy.DownloadFilesToLocation    Downloads multiple files from a Transient volume into local filenames
        specified by the caller.
        
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll    C    
        
        FSC_Download            Downloads a file from a Transient volume into a local filename specified by the caller.
        FSC_DownloadMultiple    Downloads multiple files from a Transient volume into local filenames specified by the
        caller.
        
        
        fscjavaclientproxy.jar    Java    
        
        CommonsFSCWholeFileIOImpl.download    Downloads one or more files from a Transient volume into local filenames
        specified by the caller.
        
        
        NOTE: 
        
        In 4 tier mode, you can also use any of the FSC APIs with transient file tickets. In 2 tier mode, a client must
        use one of the 4 FCC calls listed above. This list may not be complete. Several sets of wrapper classes exist
        within Teamcenter source code base which invoke these FMS operation implementations.
        
        
        3.    Visualization operations for creating the VisStructureContext from clients that initiate the launch of
        Teamcenter runtime objects like BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs'
        """
        return cls.execute_soa_method(
            method_name='createLaunchFile',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'idInfos': idInfos, 'serverInfo': serverInfo, 'userDataAgentInfo': userDataAgentInfo, 'sessionInfo': sessionInfo},
            response_cls=VVITicketsResponse,
        )

    @classmethod
    def createMarkup(cls, inputs: List[MarkupInput]) -> MarkupResponse:
        """
        This operation creates the visualization supported markup Dataset of types DirectModel3DMarkup, Markup or
        DirectModelMarkup and commits the uploaded files to the Dataset.  In addition, this operation also associates
        the newly created markup Dataset to input Teamcenter objects with relevant relations as per the data model. 
        Associated Dataset and IMANFile objects are then accessible to other authorized clients throughout the PLM
        system.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        Use cases:
        The mechanism for a client application to associate the markups to the Teamcenter objects requires several
        steps.  
        
        1.    Client application invokes the' FileManagementService::getWriteTickets' operation to obtain an FMS write
        ticket for each file / IMANFile object that will be stored in the markup Dataset. 
        
        See the Dependencies section below for details.
        
        2.    Use a FMS operation to upload the files to the PLM volume (see the Dependencies section below).  The
        client application is responsible for making sure that the FMS operation used to upload the files indicates
        that the file uploads were successful. If a file upload were to fail, then the file should not be committed. 
        This should effectively prevent the construction of database infrastructure referencing files not present in
        the volume.  Check the documentation for each operation to determine how to react to upload failures.  Some
        upload operations are defined in such a way that a single result code is returned to the caller, or a single
        exception thrown, to indicate that the bulk file upload was entirely unsuccessful.  These operations typically
        roll back all the successfully uploaded files before returning to the caller.  Some operations may accept a
        flag indicating whether uploads should continue when an error is encountered.  Other operations return a
        separate result code for each file, indicating which files were successfully uploaded to a PLM volume ahich
        uploads failed.
        
        It is the responsibility of the client application to either commit or rollback all successfully uploaded
        files.  Files that failed to upload need not be rolled back.
        
        Client responsibilities if upload of files to the volume fails:
        a.    Client applications must not invoke the 'createMarkup' visualization operation on files that did not
        successfully upload to the PLM volume.  This causes file access errors downstream, when clients attempt to
        reference (or download) the file data, which will be missing from the PLM volume.
        
        b.    In the event of a partial upload success (some files uploaded successfully, but others did not), it is
        the responsibility of the client application to determine whether it is acceptable in context to commit the
        successfully uploaded files to the PLM system.
        - If not, then there is a companion FMS operation to roll back the successfully uploaded files (see
        Dependencies section).  The rollback method removes the files from the PLM volume.
        - If so, then it is the responsibility of the client application to avoid including information corresponding
        to the files that did not successfully upload to the PLM volume in the input list passed to the 'createMarkup'
        visualization operation.
        
        
        
        3.    The client application then invokes the createMarkup visualization operation on the successfully uploaded
        files.  This step creates the visualization supported markup Dataset of types DirectModel3DMarkup, Markup,
        DirectModelMarkup and commits the uploaded files to the Dataset.  In addition, this operation also associates
        the newly created markup Dataset with input Teamcenter objects with relevant relations as per the data model. 
        Associated Dataset and IMANFile objects are then accessible to other authorized clients through the PLM system.
        If there are any failures during this operation, the client is responsible for rolling back the associated file
        from the volume through FMS operation.  Check the documentation for each operation to determine how to react to
        cleanup activity.
        
        Example:
        Markup Dataset of type DirectModel3DMarkup is created for markup files authored on 3D data (e.g.,
        DirectModel/DirectModelAssembly/VisStructureContext).
        
        Note:  In case markups are created against BOMView or BOMViewRevision or Teamcenter runtime objects like
        BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the responsibility of the
        client to create VisStructureContext object and provide VisStructureContext as an input to the operation (See
        Dependency section for service operations to use).
        
        Markup Datasets of type Markup are created for markup files created against 2D data (e.g., Image Dataset).
        
        Use Case References:
        
        1.    FMS operation for obtaining an FMS write ticket.
        
        'FileManagementService::getWriteTickets'    This operation obtains FMS write tickets for IMANFile objects 
        
        2.    FMS operations for uploading and rolling back the files to/from the PLM volume.
        
        Library                        Binding
        
        FCCClientProxy.lib                C
        
        'FCC_UploadFileToPLM'        Uploads a file to a PLM volume.
        'FCC_UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FCC_RollbackFile'                Removes a file from a PLM volume.
        'FCC_RollbackFiles'                Removes multiple files from a PLM volume.
        
        fccjavaclientproxy.jar        Java
        
        'FileCacheProxy.UploadFileToPLM'        Uploads a file to a PLM volume.
        'FileCacheProxy.UploadFilesToPLM'        Uploads multiple files to a PLM volume.
        'FileCacheProxy.RollbackFile'                Removes a file from a PLM volume.
        'FileCacheProxy.RollbackFiles'            Removes multiple files from a PLM volume.
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll        C
        
        'FSC_Upload    '                Uploads a file to a PLM volume.
        'FSC_UploadMultiple'        Uploads multiple files to a PLM volume.
        'FSC_Rollback'                 Removes a file from a PLM volume.
        'FSC_RollbackMultiple'        Removes multiple files from a PLM volume.
        
        fscjavaclientproxy.jar        Java
        
        'CommonsFSCWholeFileIOImpl.upload'    Uploads one or more files to a PLM volume.
        'CommonsFSCWholeFileIOImpl.rollback'    Removes one or more files from a PLM volume.
        
        NOTE: This list may not be complete.  Several sets of wrapper classes exist within Teamcenter source code base
        which invoke these FMS operation implementations.
        
        3.    Visualization operations for creating the VisStructureContext when the markups are created against
        Teamcenter objects like BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines from Structure
        Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs (const std::vector< Teamcenter::Soa::Internal::Visualization::_2008_06::
        StructureManagement::CreateVisSCsFromBOMsInfo > & info);' 
        
        4.    Visualization operations for creating the markup Dataset, associating the created markup Dataset and the
        input Teamcenter objects and committing the files to the markup Dataset.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::MarkupResponse createMarkup ( const
        std::vector<Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::MarkupInput >& inputs ) ;'
        """
        return cls.execute_soa_method(
            method_name='createMarkup',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=MarkupResponse,
        )

    @classmethod
    def createSessionModel(cls, inputs: List[SessionModelInput]) -> SessionModelResponse:
        """
        This operation creates the data model for the Vis_Session Dataset type and commits the uploaded files (markups,
        images etc) to the Dataset.  In addition, this operation also associates the newly created Vis_Session Dataset
        to input Teamcenter objects with relevant relations as per the data model. After initial creation, the
        'saveSession' Visualization operation is a required step and needs to be invoked to successfully upload the
        Teamcenter Visualization session file to the Vis_Session Dataset.  Saving of Teamcenter visualization session
        to Teamcenter is a two step process. It contains internal references to the other files it depends on, so they
        must be saved first via 'createSessionModel' Visualization operation (see Dependencies Section). After the
        references are persisted, then the Teamcenter visualization session file is persisted via 'saveSession'
        Visualization operation (see Dependencies Section).  Associated Dataset and ImanFile objects are then
        accessible to other authorized clients throughout the PLM system.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        Use cases:
        The mechanism for a client application to associate the session to the Teamcenter objects requires several
        steps.
        
        1.    Client application invokes the 'FileManagementService::getWriteTickets' operation to obtain an FMS write
        ticket for each file / IMANFile object that will be stored in the markup Dataset. 
        
        See the Use Case Referencion below for details.
        
        2.    Use a FMS operation to upload the files to the PLM volume (see the Use Case References section below).
        The client application is responsible for making sure that the FMS operation used to upload the files indicates
        that the file uploads were successful. If a file upload were to fail, then the file should not be committed.
        This should effectively prevent the construction of database infrastructure referencing files not present in
        the volume. Check the documentation for each operation to determine how to react to upload failures. Some
        upload operations are defined in such a way that a single result code is returned to the caller, or a single
        exception thrown, to indicate that the bulk file upload was entirely unsuccessful. These operations typically
        roll back all the successfully uploaded files before returning to the caller. Some operations may accept a flag
        indicating whether uploads should continue when an error is encountered. Other operations return a separate
        result code for each file, indicating which files were successfully uploaded to a PLM volume and which uploads
        failed.
        
        It is the responsibility of the client application to either commit or rollback all successfully uploaded
        files. Files that failed to upload need not be rolled back.
        
        Client responsibilities if upload of files to the volume fails:
        a.    Client applications must not invoke the createSessionModel visualization operation on files that did not
        successfully upload to the PLM volume. This causes file access errors downstream, when clients attempt to
        reference (or download) the file data, which will be missing from the PLM volume.
        
        b.    In the event of a partial upload success (some files uploaded successfully, but others did not), it is
        the responsibility of the client application to determine whether it is acceptable in context to commit the
        successfully uploaded files to the PLM system.
        
        If not, then there is a companion FMS operation to roll back the successfully uploaded files (see Dependencies
        section). The rollback method removes the files from the PLM volume.
        If so, then it is the responsibility of the client application to avoid including information corresponding to
        the files that did not successfully upload to the PLM volume in the input list passed to the createSessionModel
        visualization operation.
        
        
        3.    The client application then invokes the createSessionModel visualization operations on the successfully
        uploaded files. 
        This step creates the visualization supported session Dataset of type Vis_Session and commits the uploaded
        files to the Dataset. In addition, this operation also associates the newly created session Dataset with input
        Teamcenter objects with relevant relations as per the data model. Associated Dataset and IMANFile objects are
        then accessible to other authorized clients through the PLM system. If there are any failures during this
        operation, client is responsible for rolling back the associated file from the volume through FMS operations.
        Check the documentation for each API to determine how to react to cleanup activity.
        
        Note: In case session is created against BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines
        from Structure Manager or BOPLines from Manufacturing Process Planner, it is the responsibility of the client
        to create VisStructureContext object and provide VisStructureContext as an input to the operation (See Use Case
        References section for service operations to use).
        
        Use Case References:
        This operation is used in conjunction with other FileManagementService service operations, Visualization
        service operations and the FMS. Please consult the documentation for each of these available operations for
        details on the requirements, usage, and environments in which they should be used. These dependencies include:
        
        1.    FMS operation for obtaining a FMS write ticket:
        FileManagementService::getWriteTickets        This operation obtains FMS write tickets for IMANFile objects 
        
        2.    FMS methods for uploading and rolling back the files to/from the PLM volume.
        
        Library                        Binding
        
        FCCClientProxy.lib                C
        
        FCC_UploadFileToPLM        Uploads a file to a PLM volume.
        FCC_UploadFilesToPLM        Uploads multiple files to a PLM volume.
        FCC_RollbackFile                Removes a file from a PLM volume.
        FCC_RollbackFiles                Removes multiple files from a PLM volume.
        
        fccjavaclientproxy.jar            Java
        
        FileCacheProxy.UploadFileToPLM        Uploads a file to a PLM volume.
        FileCacheProxy.UploadFilesToPLM        Uploads multiple files to a PLM volume.
        FileCacheProxy.RollbackFile                Removes a file from a PLM volume.
        FileCacheProxy.RollbackFiles            Removes multiple files from a PLM volume.
        
        FSCClientProxy.dll
        FSCNativeClientProxy.dll        C
        
        FSC_Upload                    Uploads a file to a PLM volume.
        FSC_UploadMultiple        Uploads multiple files to a PLM volume.
        FSC_Rollback                Removes a file from a PLM volume.
        FSC_RollbackMultiple        Removes multiple files from a PLM volume.
        
        fscjavaclientproxy.jar            Java
        
        CommonsFSCWholeFileIOImpl.upload        Uploads one or more files to a PLM volume.
        CommonsFSCWholeFileIOImpl.rollback        Removes one or more files from a PLM volume.
        
        3.    Visualization operations for creating the VisStructureContext when the session are created against
        Teamcenter objects like BOMView, BOMViewRevision or Teamcenter runtime objects like BOMLines from Structure
        Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs '
        
        4.    Visualization operations for creating the session Dataset, associating the created Vis_Session Dataset
        and the input Teamcenter objects and committing the files to the session Dataset.
        NOTE: The createSessionModel operation should be followed by saveSession operation.
        
        Following are the list of Visualization operations needed to create Vis_Session Dataset:
        
        'Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::SessionModelResponse createSessionModel(
        Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement::SaveSessionResponse saveSession '
        
        NOTE: Please refer to Visualization service documentation for saveSession operation. 
        
        """
        return cls.execute_soa_method(
            method_name='createSessionModel',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SessionModelResponse,
        )

    @classmethod
    def createTwoDSnapshot(cls, input: List[TwoDSnapshotInput]) -> CreateTwoDSnapshotResponse:
        """
        A 2D snapshot dataset (Vis_snapshot_2D_View_Data) is an object in Teamcenter used to capture the state of a 2D
        scene including markups and viewport information.  This object is used for manufacturing planning and issue
        capture use cases.  The dataset and associated data model is designed to support change management, and updates
        to the base image without significant rework to the previously captured snapshots.  That is, the snapshots can
        be easily re-used and updated.  The data model consists of a dataset storing several files (markup layer CGM
        files, viewport xml file, thumbnail image, and asset image), a relation to the ItemRevision managing the base
        image, and a form containing attributes that can be used to search for snapshots based on certain attribute
        criteria. 
        This service creates a list of 2D snapshot Datasets and creates the TC_snap_shot_2d relation type between
        created Dataset and the ItemRevision containing the base image.   The service creates all elements of the data
        model including all relations and named references.
        
        
        Use cases:
        User creates a snapshot of a particular view of a 2D image with markups to illustrate an area of interest. 
        This snapshot is stored as a dataset in Teamcenter and later re-opened, recreating the view.  Typical use case
        steps are as follows:
            1.    User opens an ItemRevision that contains a 2D image into visualization.  Most typically this is a 2D
        drawing for a part that is to be manufactured.
            2.    User zooms and pans on the image to display the area of interest
            3.    User adds markups and annotations to the image
            4.    User saves a 2D snapshot object in Teamcenter that captures the view data for the 2D scene
            5.    System calls this service (createTwoDSnapshot) and passes the base image ItemRevision, dataset
        information (e.g. name, etc), file information to upload, and form data containing special attributes attached
        to the dataset.  The dataset is created with appropriate data model by this service.
        
        Once a 2D snapshot is created in Teamcenter, it can be opened in visualization to recreate a 2D scene.   The
        dataset can be sent to a viewer directly, or it can be displayed in the 2D snapshot gallery contained in the 2D
        viewer tab of the Manufacturing Process Planner (MPP) application.  
        """
        return cls.execute_soa_method(
            method_name='createTwoDSnapshot',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateTwoDSnapshotResponse,
        )

    @classmethod
    def authenticateUser(cls, username: str, password: str) -> bool:
        """
        This method validates that the specified username and password is valid. The call will take at a minimum 2
        seconds to complete to prevent a possible password probing attack.
        
        Use cases:
        A client can use this method to validate that the specified user and password credentials are valid. In some
        cases, there might be a need to impersonate a user without logging out from the current session. This method
        simply validated that the username and password combination is valid.
        
        Exceptions:
        >The service does not throw any exceptions but error codes will be returned in the service data on failure. 
        
        Return codes:
        10011: the specified use cannot be found.
        208016: If a required argument is not specified like the username.
        """
        return cls.execute_soa_method(
            method_name='authenticateUser',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'username': username, 'password': password},
            response_cls=bool,
        )

    @classmethod
    def getLatestFileReadTickets(cls, info: List[GetLatestFileReadTicketsInfo]) -> GetLatestFileReadTicketsResponse:
        """
        PLMXML files consumed by visualization contain file references of the form Dataset UID, IMANFile UID /
        filename.extension.  These references (often referred to as DAKIDs) can be resolved by either using the
        IMANFile UID directly, or by using the Dataset UID / named reference / original file name and
        resolving/objecting the latest version of the file from the volume.  For the visualization integrations, JT
        files may be updated constantly by CAD integration; however, persisted PLMXML files are expected to retrieve
        the latest JT file versions.  This method was created for visualization as a means to fetch the latest version
        of a JT file in bulk during PLMXML file loading.
        
        This method will find the latest named reference using the specified data in the 'GetLatestFileReadTicketsInfo'
        struct. The Dataset specified will have the named references listed. The named reference that has the matching
        original filename will be used to retrieve a read ticket. That way a Dataset can be versioned and this method
        will find the latest named reference even when the 'namedRef' object in the input points to and older version. 
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        
        Use cases:
        This method can be used when object references are stored externally and there is a possibility that the
        dataset had been updated since this reference was recorded. A typical example is a plmxml file created against
        a structure and some parts has been updated by the CAD system after creation. In order to visualize the new
        updated geometry, a ticket for the latest Dataset / named reference should be created. 
        
        PLMXML file loading use case:
        - User loads a PLMXML file that was persisted in Teamcenter or on the local file system
        - Visualization client loads the structure into memory and renders the assembly tree
        - User sets a node visible in the assembly tree
        - System gathers all the child occurrences of the specified node, and collects all the DAKIDs for all the
        children that must be loaded to service the visibility request.  If the top node was selected, a single call
        for parts referenced referenced in the structure will be made
        - The system calls the 'getLatestFileReadTickets' operation and passes all the appropriate DAKID information
        for all the child occurrence JT files that must be loaded to service the visibility request
        - The service returns the FMS file tickets
        - The system uses the FCCClientProxy to stream the files via FMS
        
        """
        return cls.execute_soa_method(
            method_name='getLatestFileReadTickets',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'info': info},
            response_cls=GetLatestFileReadTicketsResponse,
        )

    @classmethod
    def getMetaDataStamp(cls, application: str, servermode: int, ids: List[BusinessObject]) -> MetaDataStampTicketsResponse:
        """
        Metadata stamp for an object is generated based on the Business Object specified using the ids vector. The
        transient ticket for this generated file is sent back in the return data. The ticket can then be used to
        retrieve the file using a FMS service method like 'FccProxy::downloadFiles'. A Metadatastamp template is
        provided in Teamcenter containing MDS file with Teamcenter property name, default values and stamp. The
        Teamcenter site preference MetaDataStamp_template is used to find the item where the dataset template is
        stored. This file is processed and each Attribute specified will be replaced with the matching object property
        values if found. This file will be created in the transient volume and the transient ticket of the file is sent
        to client in the response data.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes).
        
        
        Use cases:
        The 'getMetaDataStamp' operation is used to create an overlay stamp mainly when printing a document from
        Teamcenter Visualization. The Teamcenter Visualization application has an integration to retrieve this file
        when printing a document opened from Teamcenter.  The MDS file created by this service can also be used by the
        VisView Convert and Print utilities.
        """
        return cls.execute_soa_method(
            method_name='getMetaDataStamp',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='DataManagement',
            params={'application': application, 'servermode': servermode, 'ids': ids},
            response_cls=MetaDataStampTicketsResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def createBOMsFromRecipes(cls, info: List[CreateBOMsFromRecipesInfo]) -> CreateBOMsFromRecipesResponse:
        """
        For each CreateBOMsFromRecipesInfo input structure, create a BOMWindow and then configures it according to the
        recipe contained in the supplied configuration object. The supplied configuration object is expected to be of
        type StructureContext, Fnd0StructureContext or ConfigurationContext and if not, then no BOMWindow will be
        created. If the configuration object is of type ConfigurationContext then the topline object must also be
        supplied; the topline object is not used when the configuration object is of type StructureContext because it
        contains its own topline reference. The BOMWindow settings that will be configured include:
        - The topline object.
        - The settings stored in the ConfigurationContext object.
        - If the configuration object is a StructureContext object that represents a manufacturing composition then the
        additional composition windows will also be created and configured.
        
        
        
        Use cases:
        When the user desires to reconstruct a BOMWindow from an object that recorded the configuration recipe of a
        previously configured BOMWindow. For example, this operation is used when sending selected BOMLines from the
        Structure Manager to Teamcenter Visualization. A VisStructureContext object is created to record the
        configuration settings of the BOMWindow of the Structure Manager as well as record the selected BOMLines as a
        list of occurrences. Teamcenter Visualization is given this VisStructureContext object which is used to create
        a BOMWindow configured the same as the BOMWindow in Structure Manger from which from which the
        VisStructureContext object was created.  This BOMWindow is then used by Teamcenter Visualization to obtain
        occurrences that can be visualized or otherwise examined.
        
        The following pruned launch use case illustrates the primary utilization of this method for launching selected
        lines to integrated standalone visualization or Lifecycle Viewer
        - User opens structure in Structure Manager (SM)/Multi Structure Manager (MSM)/Manufacturing Process Planner
        (MPP), configures it, selects some lines, and sends those lines to Lifecycle Viewer or integrated standalone
        visualization.
        - The launching client (system) calls createVisSCsFromBOMs to record the selected lines and BOM configuration
        information.
        - The visualization client receives the request to open the selected lines as an object reference to a
        VisStructureContext object.
        - The visualization client calls the createBOMsFromRecipes operation and passes the VisStructureContext object
        reference.
        - The system creates a BOMWindow and configures it properly (to match launching configuration).
        - The client pulls the occurrence_list property from the VisStructureContext object.
        - The client issues an expandPSFromOccurrenceList call to load the occurrences.
        - The client(system) loads the selected occurrences into the visualization client, the structure is pruned to
        contain only the occurrences sent.
        
        
        Use Case References: 
        The createBOMsFromRecipes operation acts upon previously persisted configuration objects, such as a
        VisStructureContext object.
        
        The following services are used in conjunction with createBOMsFromRecipes to complete the use case above.
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. CreateVisSCsFromBOMs
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. AreRecipesMergible
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. expandPSFromOccurrenceList
        """
        return cls.execute_soa_method(
            method_name='createBOMsFromRecipes',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateBOMsFromRecipesResponse,
        )

    @classmethod
    def createVisSCsFromBOMs(cls, info: List[CreateVisSCsFromBOMsInfo]) -> CreateVisSCsFromBOMsResponse:
        """
        This operation takes a list of BOMLines (the occurrences list) and returns the VisStructureContext objects
        representing the configuration state of the BOMWindow (referred to as the configuration recipe). This
        configuration includes:
        - The occurrence UID chains for the input/selected BOMLines up to but not including the top line. 
        - Optional IMANFile reference to the PLMXML static representation of the BOMWindow.
        
        
        
        This service supports both the interoperation of selected BOMLines from the Teamcenter Rich Client to
        Teamcenter Visualization and also the capture/persistence of the configuration recipe for a particular
        BOMWindow. The occurrence list records the selected BOMLines at the time of interoperation and can be used in
        later operations to populate/expand a BOMWindow with those same occurrences (see 'expandPSFromOccurrenceList'
        operation for more information).
        
        Use cases:
        When the user desires to create a persistent object that records the configuration recipe of a particular
        BOMWindow. The resulting VisStructureContext object would assumedly be used to later reconstruct a BOMWindow
        with the same configuration recipe and the recorded UID occurrence chains would be used to populate/expand the
        constructed BOMWindow with specific BOMLines. For example, this operation is used when sending selected
        BOMLines from the Structure Manager to Teamcenter Visualization and also to capture the configuration recipe
        for storage in Vis Sessions.
        
        Visualization pruned launch use case
        - User opens a structure in Structure Manager (SM)/Multi Structure Manager (MSM)/Manufacturing Process Planner
        (MPP), and configures it
        - User selects some lines they want to send to visualization as a pruned structure
        - System calls 'createVisSCsFromBOMs' to record the selected lines and the configuration of the BOM to send
        
        
        
        Visualization session save use case
        - 1.    User performs Visualization pruned launch use case and loads occurrences into visualization
        - 2.    User creates some authored visualization content (e.g. snapshots, motions, etc)
        - 3.    User saves session to Teamcenter
        - 4.    System calls 'createRecipesFromBOMs' operation to capture the configuration and any pruning information
        as a VisStructureContext object.  UID of object returned.
        - 5.    System writes the VisStructureContext object reference into the visualization session data
        - 6.    System saves the visualiation session dataset to Teamcenter, and relates it to the VisStructureContext
        object created by the service
        
        
        
        Visualization Technical Illustration and 3D Markup save use cases
        Similar to session save use case, except saving a different data type.  Uses this service to create the recipe
        for the authored visualization data in the Teamcenter data model.
        
        Use Case Dependencies: 
        The 'createVisSCsFromBOMs' operation is called with input BOMLines from an existing BOM Window. Therefore, the
        BOMWindow must have already been created and populated with at least a top line.
        
        The following services are used in conjunction with this service to complete the use cases above.
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'createBOMsFromRecipes'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'areRecipesMergible'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'expandPSFromOccurrenceList'
        Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement. 'createSessionModel'
        Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement. 'saveSession'
        Teamcenter::Soa::Internal::Visualization::_2008_06::DataManagement. 'updateSessionModel'
        """
        return cls.execute_soa_method(
            method_name='createVisSCsFromBOMs',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateVisSCsFromBOMsResponse,
        )

    @classmethod
    def expandPSFromOccurrenceList(cls, info: List[ExpandPSFromOccurrenceListInfo], pref: ExpandPSFromOccurrenceListPref) -> ExpandPSFromOccurrenceListResponse:
        """
        This operation returns BOMLines for the occurrences recorded in the occurrence list of the input occurrence
        object. Optionally, it can also return the objects of given type and relation that are attached to the objects
        that the BOMLines represent ('objectofBOMLine'). 
        
        The operation can expand datasets and other objects related to parent and child BOMLines.  Expansion of the
        related objects can be controlled by specifying a filter. The filter criteria supported are: relation name,
        related object type, and named references. 
        
        This operation allows for expansion to reference object associated to a named reference.  Typically this is a
        file and in that case a FMS ticket will be returned to provide access to this file. Where a named reference
        points to a file, this operation allows caller to specify from a defined set of handler options, which specific
        handler should be used in choosing file(s) to return.  This is specified through the input parameter
        'NamedRefHandler' (included in the info object).
        
        
        Use cases:
        When the user wants to expand a specific list of occurrences into an existing BOMWindow that contains the
        parent BOMLine of the occurrences.
        This service is used to support the following primary use cases.
        
        Pruned launch of selected lines to visualization
        - User opens structure in Structure Manager/Multi Structure Manager/Manufacturing Process Planner, configures
        it, selects some lines, and sends those lines to Lifecycle Viewer or integrated standalone visualization
        - The launching client (system) calls 'createVisSCsFromBOMs' to record the selected lines and BOM configuration
        information
        - The visualization client receives the request to open the selected lines as an object reference to a
        VisStructureContext object
        - The visualization client calls the 'createBOMsFromRecipes' operation and passes the VisStructureContext
        object reference
        - The system creates a BOMWindow and configures it properly (to match launching configuration)
        - The client pulls the occurrence_list from the VisStructureContext object
        - The client issues an 'expandPSFromOccurrenceList' call to load the occurrences
        - The client(system) loads the selected occurrences in the visualization client, the structure is pruned to
        contain only those occurrences sent
        
        
        
        Product View launch to visualization
        - 1.    User selects a Product View in the Product View Gallery of embedded visualization inside Structure
        Manager/Multi Structure Manager/Manufacturing Process Planner, or in My Teamcenter and sends it to integrated
        standalone TcVis or Lifecycle Viewer
        - 2.    The visualization client receives the request to open the selected product View
        - 3.     The visualization client interrogates the Product View data model, fetches the files from the dataset,
        and gets a list of visible lines for the Product View
        - 4.    The client issues an 'expandPSFromOccurrenceList' call for all visible lines referenced by the Product
        View
        - 5.    The client loads the visible lines then applies the product view
        
        
        
        Use Case Dependencies: 
        The 'expandPSFormOccurrenceList' operation is called for existing parent BOMLines within a BOMWindow (i.e. the
        parent BOMLine(s) must already exist within a created BOMWindow) and also requires a previously
        captured/defined list of occurrences represented as a list of UID strings.
        
        The following services are used in conjunction with 'expandPSFromOccurrenceList' to complete the use cases
        above.
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'createVisSCsFromBOMs'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'createBOMsFromRecipes'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'areRecipesMergible'
        Teamcenter::Soa::Internal::Visualization::_2010_09::DataManagement. 'getSnapshot3DInfo'
        """
        return cls.execute_soa_method(
            method_name='expandPSFromOccurrenceList',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info, 'pref': pref},
            response_cls=ExpandPSFromOccurrenceListResponse,
        )

    @classmethod
    def areRecipesMergable(cls, info: List[AreRecipesMergableInfo]) -> AreRecipesMergableResponse:
        """
        This operation takes a list of configuration object/BOMWindow pairs and determines if their configuration
        recipes are equivalent. That is, the configuration of the BOMWindow is compared to the configuration recipe
        recorded in the configuration object. For the configurations to be considered equivalent the following must all
        be true:
        - The top line recorded in the configuration object must match the top line of the BOMWindow.
        - The RevisionRule recorded in the configuration object must be equivalent to the RevisionRule of the BOMWindow.
        - If the BOMWindow has variant options set on the top line then the configuration object must have the same
        variant option names and the option values must match those in the BOMWindow.
        - If the BOMWindow has an active assembly arrangement then the configuration object must have the same assembly
        arrangement object recorded.
        
        
        
        This service supports use cases involving the interoperation of selected BOMLines from the Teamcenter Rich
        Client to Teamcenter Visualization. The occurrences list of the configuration object records the selected
        BOMLines at the time of interoperation. The occurrences list is then be used in a later operation to populate a
        BOM Window with those same occurrences (see 'expandPSFromOccurrencesList' operation for more information). This
        operation is used to determine if the selected lines in the new interoperation request can be merged (opened
        within) an existing document in Teamcenter Visualization opened as a result of a previous interoperation.
        
        Use cases:
        A Teamcenter Visualization client has a configured BOMWindow from a previous interoperation with Teamcenter.
        That client then receives a new request to open a configuration object. The client may use this service to
        determine if the occurrences recorded in the new configuration object may be merged into its existing BOMWindow.
        The basic user flow where this service is utilized by visualization is as follows:
        - User opens Structure Manager/Multi Structure Manager/Manufacturing Process Planner, configures the structure,
        selects some lines, and sends those lines to Lifecycle Viewer or standalone Teamcenter Visualization.
        - The viewer client opens the structure, configures it as it was in the launching client, and then expands and
        loads the selected occurrences that were sent.
        - User goes back to Structure Manager/Multi Structure Manager/Manufacturing Process Planner client, selects a
        few more occurrences to also send and merge into the open structure inside visualization, and sends those lines
        to Lifecycle Viewer or standalone Teamcenter Visualization.
        - Visualization receives the merge request from the user, and the system calls the 'areRecipesMergible' service
        to determine if the specified occurrences can be merged into an already open window in the visualization
        client. If mergable, the visualization client merges the occurrences into an existing open structure.
        
        
        
        Use Case References: 
        There must be a previously existing configured BOM Window and an existing configuration object.
        
        The following services are used in conjunctions with areRecipesMergible to complete the use case above.
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'createVisSCsFromBOMs'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'createBOMsFromRecipes'
        Teamcenter::Soa::Internal::Visualization::_2008_06::StructureManagement. 'expandPSFromOccurrenceList'
        """
        return cls.execute_soa_method(
            method_name='areRecipesMergable',
            library='Internal-Visualization',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=AreRecipesMergableResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import ListOfValues
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttachedPropDescsResponse(TcBaseObj):
    """
    Attached Property Descriptors Response
    
    :var inputTypeNameToPropDescOutput: Map of input type name to PropertyDescriptor
    :var serviceData: ServiceData which has output tags as plain objects and errors in partialError
    """
    inputTypeNameToPropDescOutput: InputTypeNameToPropDescOutputMap = None
    serviceData: ServiceData = None


@dataclass
class PropDesc(TcBaseObj):
    """
    The PropDesc struct describes information about the Teamcenter property
    
    :var propName: Name of the property
    :var displayName: Display name of the property
    :var isEnabled: Specifies whether the property is enabled
    :var isModifiable: Specifies whether the property is modifiable
    :var attachedSpecifier: attachedSpecifier
    :var maxLength: maxLength
    :var interdependentProps: interdependentProps
    :var defaultValue: Default value for the property
    :var propValueType: Value type for the property, PROP_untyped (0) No Value type PROP_char (1) Value is a single
    character PROP_date (2) Value is a date PROP_double (3) Value is a double PROP_float (4) Value is a float PROP_int
    (5) Value is an integer PROP_logical (6) Value is a logical PROP_short (7) Value is a short PROP_string (8) Value
    is a character string PROP_typed_reference (9) Value is a typed reference PROP_untyped_reference (10) Value is an
    untyped reference PROP_external_reference (11) Value is an external reference PROP_note (12) Value is a note
    PROP_typed_relation (13) Value is a typed relation PROP_untyped_relation (14) Value is an untyped relation
    :var propType: Type for the property PROP_unknown (0) Property type is Unknown PROP_attribute (1)  Based on a POM
    Attribute (int, string, ...) PROP_reference (2)  Based on a POM Reference PROP_relation (3) Based on an
    ImanRelation PROP_compound (4) Based on a property from another Type PROP_runtime (5) Based on a computed value
    :var isDisplayable: isDisplayable
    :var isArray: Specifies whether the property is an array or single value
    :var maxNumElems: Specifies the max number of elements
    :var lov: ListOfValues object attached to the property (if any)
    :var isRequired: Specifies whether the property is required
    """
    propName: str = ''
    displayName: str = ''
    isEnabled: bool = False
    isModifiable: bool = False
    attachedSpecifier: int = 0
    maxLength: int = 0
    interdependentProps: List[str] = ()
    defaultValue: str = ''
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    maxNumElems: int = 0
    lov: ListOfValues = None
    isRequired: bool = False


@dataclass
class PropDescInfo(TcBaseObj):
    """
    PropDescInfo structure represents all the Property Names and associated type to retrieve Property Descriptors.
    
    :var typeName: The name of the type associated with the input Property Names.
    :var propNames: A list of the input Property Names to retrieve Property Descriptors.
    """
    typeName: str = ''
    propNames: List[str] = ()


@dataclass
class PropDescOutput(TcBaseObj):
    """
    Property Description Output
    
    :var propName: Input Property name for which PropDesc needs to be fetched
    :var propertyDesc: The PropDescriptor struct describes information about the Teamcenter property
    """
    propName: str = ''
    propertyDesc: PropDesc = None


"""
InputTypeNameToPropDescOutputMap
"""
InputTypeNameToPropDescOutputMap = Dict[str, List[PropDescOutput]]
